#!/bin/sh

#
# Netborder Express Installation script
# Copyright (c) 2009, Sangoma Technologies Inc.
# ---------------------------------------------
# Feb 25, 2009  Moises Silva  Initial version
#

NBE_ACTION=$1
NBE_MAIN_PREFIX=/usr/local
NBE_SYS_PREFIX=/etc
NBE_INSTALL_WANPIPE="YES"
NBE_DEBUG="NO"
NBE_ROOT_DIR=`pwd`

function printdbg()
{
  if [ "$NBE_DEBUG" = "YES" ]
  then
    echo $1;
  fi
}

function printmsg()
{
  echo $1
}

function printerr()
{
  echo $1
}

function install_sanity_check()
{
  if [ ! -d "$NBE_ROOT_DIR/sangoma" ] || [ ! -d "$NBE_ROOT_DIR/nbe" ]
  then
    printerr "Current directory $NBE_ROOT_DIR does not look like the installer directory! (sangoma and/or nbe directories missing)"
    exit 1
  fi
  if [ "$NBE_INSTALL_WANPIPE" = "YES" ] && [ "$(id -u)" != "0" ]
  then
    printerr "This script must be run as root or run with --nowanpipe!"
    exit 1
  fi
}

function usage()
{
  cat <<ENDOFTEXT 
  USAGE:
  $0 {action}
  or
  $0 {action} {options}

  actions:
  install
  remove

  options:
  --prefix=<prefix path>
  --nowanpipe: Do not build and install Sangoma Wanpipe Driver


ENDOFTEXT
  exit 1
}

function valid_action_no_usage()
{
# check for a valid action
# for each of these actions a function execute_action_<action-name> should exist
  printdbg "validating action $1"
  case $1 in
  install)
    printdbg "valid action $1"
    return 0 
    ;;
  remove)
    printdbg "valid action $1"
    return 0;
    ;;
  *)
    printdbg "invalid action $1"
    return 1;
    ;;
  esac
  return 1;
}

function valid_action()
{
  valid_action_no_usage $1 || usage
}

function install_netborder()
{
  printmsg "Installing Netborder Express in $NBE_MAIN_PREFIX/netborder ..."
  set -x
  mkdir -p $NBE_MAIN_PREFIX
  mkdir -p $NBE_SYS_PREFIX/init.d
  mkdir -p $NBE_SYS_PREFIX/sysconfig
  cp -r $NBE_ROOT_DIR/nbe/netborder $NBE_MAIN_PREFIX/ 
  cp $NBE_ROOT_DIR/nbe/netborder.conf $NBE_SYS_PREFIX/sysconfig/
  cp $NBE_ROOT_DIR/nbe/netborder-gateway $NBE_SYS_PREFIX/init.d/
  cp $NBE_ROOT_DIR/nbe/netborder-webconfig $NBE_SYS_PREFIX/init.d/
  set +x
  printmsg "Done!"
  return 0
}

function remove_netborder()
{
  printmsg "Removing Netborder Express from $NBE_MAIN_PREFIX"
  set -x
  rm -rf $NBE_MAIN_PREFIX/netborder
  rm -rf $NBE_SYS_PREFIX/init.d/netborder-gateway
  rm -rf $NBE_SYS_PREFIX/sysconfig/netborder.conf
  set +x
}

function remove_wanpipe()
{
  if [ "$NBE_INSTALL_WANPIPE" = "YES" ]
  then
    printmsg "Wanpipe drivers require manual removal"
    # TODO: ask Nenad for removal instructions
  fi
}

function install_wanpipe()
{
  if [ "$NBE_INSTALL_WANPIPE" = "YES" ]
  then
    printmsg "Installing Wanpipe Drivers ..."
    cd sangoma/wanpipe/
    ./Setup drivers --protocol=AFT_TE1 --silent &> $NBE_ROOT_DIR/wanpipe.log
    if [ $? != 0 ]
    then
      printerr "Wanpipe installation failed!, check wanpipe.log"
      cd -
      return 1
    fi
    printmsg "Done!"
  fi
  return 0
}

function parse_opts()
{
  printdbg "checking args"
  for arg in $@
  do
    case $arg in
    --prefix*)
      NBE_MAIN_PREFIX=`echo $arg | cut -d'=' -f2`
      ;;
    --sysprefix*)
      NBE_SYS_PREFIX=`echo $arg | cut -d'=' -f2`
      ;;
    --nowanpipe*)
      NBE_INSTALL_WANPIPE="NO"
      ;;
    --debug*)
      NBE_DEBUG="YES"
      ;;
    *)
      valid_action_no_usage $arg
      if [ $? = 1 ]
      then
        printerr "invalid option $arg" 
        usage
        exit
      fi
      ;;
    esac
  done
  return 0
}

function execute_action_install()
{
  rm -f $NBE_ROOT_DIR/installenv.sh
  install_netborder
  install_wanpipe
  nbe_env=$(set | grep NBE_)
  echo $nbe_env > $NBE_ROOT_DIR/installenv.sh
}

function execute_action_remove()
{
  if [ ! -f "$NBE_ROOT_DIR"/installenv.sh ]
  then
    printerr "It seems you have not run install before: (no installenv.sh file found)"
    exit 1
  fi
  source $NBE_ROOT_DIR/installenv.sh
  remove_netborder
  remove_wanpipe
}

################### main() ########################

# check a valid action is provided
valid_action $NBE_ACTION 

# verify valid options are provided and set the proper variables
parse_opts $@ 

# check we are wherever we are supposed to be and with the proper user etc
install_sanity_check 

# execute the action
execute_action_$NBE_ACTION



