# Helper methods around auto-generated pstnconfig module

# Standard modules
import sys

# Impl. modules
import pstnconfig as m

##############################################################################
# Helpers for registering IDs for model entities used in PSTN Editor
#
# TODO put in a class/belongs to the controller
#
# ExtJS Tree items must have a unique ID - also used to retrieve associated 
#   object
# ID generator using a 1-based count() iterator

import itertools
idgen = itertools.count(1)

# Map of unique id/model entities 
_idToModelEntityDict = {}

# Register object using an ID for later edition from TreeView of Editor
# @returns id
def registerObjectForEditor(o):
  # If the object exists in the list, return its current id instead of creating
  # a new one.
  if o in _idToModelEntityDict.values():
    return getObjectId(o)
  else:
    id = idgen.next()
    _idToModelEntityDict[id] = o
    return id  

# Scans the object dictionary _idToModelEntityDict to find the parameter object's
# id.
def getObjectId(o):
  for k,v in _idToModelEntityDict.items():
    if v == o:
      return k

# Get object from Id assigend in Editor tree
def getObjectFromEditorId(id):
  # Converted to int as passed as str to controller method
  return _idToModelEntityDict[int(id)]

# Deletes an object from the dicitonary
def deleteObjectFromEditor(o):
  del _idToModelEntityDict[getObjectId(o)]

def getNextBoardID(pstnConfig):
  id = 0;
  for board in pstnConfig.get_boards().get_sangomaAnalogBoard():
    if board.get_ID() > id:
      id = board.get_ID()
  for board in pstnConfig.get_boards().get_sangomaDigitalBoard():
    if board.get_ID() > id:
      id = board.get_ID()

  return id + 1

def getNextPhysConfID(pstnConfig):
  id = 0;
  for config in pstnConfig.get_physicalInterfaceConfigs().get_sangomaAnalogConfig():
    if config.get_ID() > id:
      id = config.get_ID()
  for config in pstnConfig.get_physicalInterfaceConfigs().get_sangomaDigitalConfig():
    if config.get_ID() > id:
      id = config.get_ID()

  return id + 1

def getNextISDNConfID(pstnConfig):
  id = 0;
  isdn = pstnConfig.get_callControl().get_isdn()
  if isdn:
    for config in isdn.get_fasConfig():
      if config.get_ID() > id:
        id = config.get_ID()
    for config in isdn.get_nfasGroup():
      if config.get_ID() > id:
        id = config.get_ID()
  
  return id + 1

def getNextAnalogConfID(pstnConfig):
  id = 0;
  analog = pstnConfig.get_callControl().get_analog()
  if analog:
    for config in analog.get_analogFxoConfig():
      if config.get_ID() > id:
        id = config.get_ID()
    for config in analog.get_analogFxsConfig():
      if config.get_ID() > id:
        id = config.get_ID()

  return id + 1

###############################################################################
# Info for each supported board model
boardsModelInfo = { \
    'A101': {'type': 'digital', 'numInterfaces': 1},
    'A102': {'type': 'digital', 'numInterfaces': 2},
    'A104': {'type': 'digital', 'numInterfaces': 4},
    'A108': {'type': 'digital', 'numInterfaces': 8},
    'A200': {'type': 'analog'},
    'VIRTUAL_DIGITAL': {'type': 'virtual_digital'},
    }

###############################################################################
# Default settings for some configuration objects

defaultEchoCancellation = {
    'typexx':'HARDWARE',
    'mode':'NORMAL',
    'comfortNoiseMode':'NORMAL',
    'tailDisplacementInMs':'0',
    'doubleTalkBehavior':'OPTIMAL'
    }
defaultAcousticEchoCancellation = {
    'typexx':"HARDWARE", 'mode':"NORMAL"
    }
defaultLevelControl = {
    'typexx':"HARDWARE",
    'direction':"ALL",
    'mode':"AUTOMATIC",
    'targetLevelIndBM0':"-20",
    }
defaultBackgroundNoiseAttenuation = {
    'typexx':"HARDWARE",
    'direction':"FROM_PSTN",
    'mode':"ENABLED",
    'attenuationIndB':"-18"
    }
defaultDTMFRemoval = {
    'typexx':"HARDWARE",
    'direction':"FROM_PSTN",
    'mode':"DISABLED"
    }

defaultInboundParams = {
  'connect':{
    'condition':'ON-OUTBOUNDSIDE-CONNECTED'
  },
  'callerID':{
    'typexx':'Bellcore-FSK-SDMF-MDMF'
  },
  'notifyOutboundSide':{
    'condition':'ON-CALLER-ID',
    'delayInMs':0
  }
}

defaultOutboundParams = {
  'dial':{
    'DTMFDigitDurationInMs':200,
    'interDigitDelayInMs':100,
    'active':'YES',
    'waitForDialTone':'YES',
    'preDialDelayInMs':100
  },
  'connectOutboundSide':{
    'condition':'AFTER-DIALING-DESTINATION-NUMBER'
  },
  'connect':{
    'condition':'ON-OUTBOUNDSIDE-CONNECTED',
    'delayInMs':0
  }
}

defaultDisconnectParams = {
  'ringCadenceDetector':{
    'active':'YES',
    'ringCadenceOffDelayMs':7000
  },
  'detectors':[
    {
      'name':'peerHangup',
      'sequence':'dial'
    },
    {
      'name':'disconnect-tone',
      'sequence':'busy'
    }
  ]
}

defaultT1Config = {
  'lineEncoding':'B8ZS',
  'clocking':'TERMINAL',
  'loopbackMode':'DISABLED',
  'framing':'ESF',
  'LBO':'0dB'
}
        
defaultAudioParams = {
  'defaultPcmLaw':'PCMU',
  'voicePacketLengthInMs':'20',
  'toneSetName':'us'
}

defaultFasConfig = {
  'displayName':'TODO',
  'termination':'TERMINAL',
  'switchVariant':'5ESS',
  'BChannelNegotiation':'EXCLUSIVE',
  'initiateRestartProcedure':'YES',
  'inbandProgressTonesGeneration':'AS-NEEDED',
  'inbandProgressTonesIndicator':'PROGRESS'
}


###############################################################################
# Writes the pstnConfig as XML to a "file" subclass
def writeConfigToFileAsXML(pstnConfig, outfile):
  outfile.write('<?xml version="1.0" ?>\n')
  pstnConfig.export(outfile, 0)

###############################################################################
# Creates a pstnConfig object with defaults for Sangoma, including a 
#   default BIDIR channel group
# @returns pstnConfig object 
def createDefaultSangomaPstnConfig():
  # providers
  bChannelsThreads = m.BChannelsThreads.factory(
      count=4, priority="TIME_CRITICAL")
  dChannelsThreads = m.DChannelsThreads.factory(
      count=1, priority="HIGHEST")
  sangoma = m.sangoma.factory(BChannelsThreads=bChannelsThreads,
      DChannelsThreads=dChannelsThreads)
  providers = m.providers.factory(sangoma=sangoma)
  # callControl
  channelGroup = m.channelGroup.factory(ID="default", direction="BIDIR",
      outboundHuntingScheme="REVERSE_LINEAR")
  # NOTE: the "channelGroup" property of the generated "channelGroups" class
  #   is a - list - of "channelGroup" objects
  channelGroups = m.channelGroups.factory(channelGroup=[channelGroup])
  callControl = m.callControl.factory(channelGroups=channelGroups)
  # pstncOnfig
  return m.pstnConfig.factory(version='2.0',
      providers=providers,
      boards=m.boards.factory(),
      physicalInterfaceConfigs=m.physicalInterfaceConfigs.factory(),
      callControl=callControl)

###############################################################################
# TODO add factory/context where next wanpipe, assigned ids are maintained
# @returns Sangoma Digital Board (already added to the PSTN Config)
def addSangomaDigitalBoard(pstnConfig, boardModel, boardID, starting_wanpipe):

  interfaceList = []
  for i in range(1, boardsModelInfo[boardModel]['numInterfaces'] + 1):
    # TODO factory/context to know which id/wanpipe to use
    interfaceList.append(m.interface.factory(ID=i, 
      displayName='B%uI%u' % (boardID, i), 
      wanpipe=starting_wanpipe+i-1))

  sangomaDigitalBoard = m.sangomaDigitalBoard.factory(
      ID=boardID, 
      displayName='B%u - %s_%s' % (boardID, boardModel, 'digital'),
      model=boardModel,
      interface=interfaceList) # "interface" here is a list
  
  pstnConfig.get_boards().add_sangomaDigitalBoard(sangomaDigitalBoard)

  return sangomaDigitalBoard

###############################################################################
# TODO add factory/context where next wanpipe, assigned ids are maintained
# @returns Sangoma Digital T1 Config (already added to the PSTN Config)
def addT1SangomaDigitalConfig(pstnConfig,displayName,configParams,audioParams, \
    EchoCancellation = defaultEchoCancellation,
    AcousticEchoCancellation = defaultAcousticEchoCancellation,
    LevelControl = defaultLevelControl,
    BackgroundNoiseAttenuation = defaultBackgroundNoiseAttenuation,
    DTMFRemoval = defaultDTMFRemoval):

  # Add DChannel config to the list
  configParams['DChannelT1'] = \
    m.DChannelT1.factory(mtuSize='2048', channelIndex=24)

  # Create a digitalT1 from the parameter list
  digitalT1 = m.digitalT1.factory(**configParams)

  # Create the audio element and all subElement
  echoCancellation = m.EchoCancellation.factory(**EchoCancellation)
  acousticEcho = m.AcousticEchoCancellation.factory(**AcousticEchoCancellation)
  levelControl = m.LevelControl.factory(**LevelControl)
  backgroundNoise = m.BackgroundNoiseAttenuation.factory(**BackgroundNoiseAttenuation)
  dtmfRemoval = m.DTMFRemoval.factory(**DTMFRemoval)

  audioParams['EchoCancellation'] = echoCancellation
  audioParams['AcousticEchoCancellation'] = acousticEcho
  audioParams['LevelControl'] = levelControl
  audioParams['BackgroundNoiseAttenuation'] = backgroundNoise
  audioParams['DTMFRemoval'] = dtmfRemoval
  audio = m.audio.factory(**audioParams)

  # Create the digital config
  sangomaDigitalConfig = m.sangomaDigitalConfig.factory()

  # Set the displayName to the given name and assign the digitalT1
  # and the audioConfig we just created to the config.
  sangomaDigitalConfig.set_displayName(displayName)
  sangomaDigitalConfig.set_digitalT1(digitalT1)
  sangomaDigitalConfig.set_audio(audio)
  sangomaDigitalConfig.set_ID(getNextPhysConfID(pstnConfig))

  # Finally add the whole package to physical configuration list of the current
  # configuration.
  pstnConfig.get_physicalInterfaceConfigs().add_sangomaDigitalConfig(sangomaDigitalConfig)

  return sangomaDigitalConfig

# This method is exactly identical to addT1SangomaDigitalConfig except that the 
# work is done for an E1 interface.
# @returns Sangoma Digital E1 Config (already added to the PSTN Config)
def addE1SangomaDigitalConfig(pstnConfig,displayName,configParams,audioParams, \
    EchoCancellation = defaultEchoCancellation,
    AcousticEchoCancellation = defaultAcousticEchoCancellation,
    LevelControl = defaultLevelControl,
    BackgroundNoiseAttenuation = defaultBackgroundNoiseAttenuation,
    DTMFRemoval = defaultDTMFRemoval):

  configParams['DChannelE1'] = \
    m.DChannelE1.factory(mtuSize='2048', channelIndex=16)

  digitalE1 = m.digitalE1.factory(**configParams)

  echoCancellation = m.EchoCancellation.factory(**EchoCancellation)
  acousticEcho = m.AcousticEchoCancellation.factory(**AcousticEchoCancellation)
  levelControl = m.LevelControl.factory(**LevelControl)
  backgroundNoise = m.BackgroundNoiseAttenuation.factory(**BackgroundNoiseAttenuation)
  dtmfRemoval = m.DTMFRemoval.factory(**DTMFRemoval)

  audioParams['EchoCancellation'] = echoCancellation
  audioParams['AcousticEchoCancellation'] = acousticEcho
  audioParams['LevelControl'] = levelControl
  audioParams['BackgroundNoiseAttenuation'] = backgroundNoise
  audioParams['DTMFRemoval'] = dtmfRemoval
  audio = m.audio.factory(**audioParams)

  sangomaDigitalConfig = m.sangomaDigitalConfig.factory()

  sangomaDigitalConfig.set_displayName(displayName)
  sangomaDigitalConfig.set_digitalE1(digitalE1)
  sangomaDigitalConfig.set_audio(audio)
  sangomaDigitalConfig.set_ID(getNextPhysConfID(pstnConfig))

  pstnConfig.get_physicalInterfaceConfigs().add_sangomaDigitalConfig(sangomaDigitalConfig)

  return sangomaDigitalConfig

###############################################################################
# TODO add factory/context where next wanpipe, assigned ids are maintained
# Config for ISDN span not in an NFAS group
# @returns FAS Config (already added to the PSTN Config)
def addFasConfig(pstnConfig, **configParams):

  isdn = pstnConfig.get_callControl().get_isdn()
  if isdn is None:
    isdn = m.isdn.factory()
    pstnConfig.get_callControl().set_isdn(isdn)

  fasConfig = m.fasConfig.factory(**configParams)
  fasConfig.set_ID(getNextISDNConfID(pstnConfig))

  applyTo = m.applyTo.factory()
  fasConfig.set_applyTo(applyTo)

  isdn.add_fasConfig(fasConfig)
  
  return fasConfig

###############################################################################
# TODO add factory/context where next wanpipe, assigned ids are maintained
# Config for ISDN span not in an NFAS group
# @returns FAS Config (already added to the PSTN Config)
def addNfasGroup(pstnConfig, **configParams):

  isdn = pstnConfig.get_callControl().get_isdn()
  if isdn is None:
    isdn = m.isdn.factory()
    pstnConfig.get_callControl().set_isdn(isdn)

  nfasGroup = m.nfasGroup.factory(**configParams)
  nfasGroup.set_ID(getNextISDNConfID(pstnConfig))

  members = m.members.factory()
  nfasGroup.set_members(members)

  isdn.add_nfasGroup(nfasGroup)

###############################################################################
# TODO add factory/context where next wanpipe, assigned ids are maintained
# @returns Sangoma Analog Board (already added to the PSTN Config)
def addSangomaAnalogBoard(pstnConfig, boardModel, boardID, wanpipe, numInterfaces):
  interfaceList = []
  for i in range(1, numInterfaces + 1):
    # TODO factory/context to know which id/wanpipeInterface to use
    # NOTE typexx is used instead of "type" by generatorDS to avaoid reserved
    #   Python keyword
    interfaceList.append(m.interface.factory(ID=i, 
      displayName='B%uI%u' % (boardID, i), 
      typexx='FXO', wanpipeInterface=i-1))

  sangomaAnalogBoard = m.sangomaAnalogBoard.factory(
      ID=boardID, 
      wanpipe=wanpipe,
      displayName='B%u - %s_%s' % (boardID, boardModel, 'analog'),
      model=boardModel,
      interface=interfaceList) # "interface" here is a list
  
  pstnConfig.get_boards().add_sangomaAnalogBoard(sangomaAnalogBoard)

  return sangomaAnalogBoard

###############################################################################
# TODO add factory/context where next wanpipe, assigned ids are maintained
# audioParams are "audio" element factory arguments
# @returns Sangoma Analog Config (already added to the PSTN Config)
def addSangomaAnalogConfig(pstnConfig,displayName,variant,audioParams,
    EchoCancellation = defaultEchoCancellation,
    AcousticEchoCancellation = defaultAcousticEchoCancellation,
    LevelControl = defaultLevelControl,
    BackgroundNoiseAttenuation = defaultBackgroundNoiseAttenuation,
    DTMFRemoval = defaultDTMFRemoval):

  sangomaAnalogConfig = m.sangomaAnalogConfig.factory()

  echoCancellation = m.EchoCancellation.factory(**EchoCancellation)
  acousticEcho = m.AcousticEchoCancellation.factory(**AcousticEchoCancellation)
  levelControl = m.LevelControl.factory(**LevelControl)
  backgroundNoise = m.BackgroundNoiseAttenuation.factory(**BackgroundNoiseAttenuation)
  dtmfRemoval = m.DTMFRemoval.factory(**DTMFRemoval)

  audioParams['EchoCancellation'] = echoCancellation
  audioParams['AcousticEchoCancellation'] = acousticEcho
  audioParams['LevelControl'] = levelControl
  audioParams['BackgroundNoiseAttenuation'] = backgroundNoise
  audioParams['DTMFRemoval'] = dtmfRemoval
  audio = m.audio.factory(**audioParams)

  sangomaAnalogConfig.set_audio(audio)
  analog = m.analog.factory(variant=variant)
  sangomaAnalogConfig.set_displayName(displayName)
  sangomaAnalogConfig.set_analog(analog)
  sangomaAnalogConfig.set_ID(getNextPhysConfID(pstnConfig))
  pstnConfig.get_physicalInterfaceConfigs().\
      add_sangomaAnalogConfig(sangomaAnalogConfig)
      
  return sangomaAnalogConfig

###############################################################################
# @returns Sangoma Ananog Fxo Config (already added to the PSTN Config)
def addAnalogFxoConfig(pstnConfig, configParams, 
    inboundCallParams=defaultInboundParams,
    outboundCallParams=defaultOutboundParams,
    disconnectSupervisionParams=defaultDisconnectParams):
  # get analog from callControl
  analog = pstnConfig.get_callControl().get_analog()
  if analog is None:
    analog = m.analog.factory()
    pstnConfig.get_callControl().set_analog(analog)

  # create inboundCallHandling
  inboundCall = m.inboundCallHandling.factory()
  #   create connect :
  #     - delayInMs
  #     - condition
  connect = m.connect.factory(**inboundCallParams['connect'])
  #   create callerID
  #     - typexx : Bellcore-FSK-SDMF-MDMF is the only supported yet.
  callerID = m.callerID.factory(**inboundCallParams['callerID'])
  #   create notifyOutboundSide
  #     - numberOfRings
  #     - delayInMs
  #     - condition
  notifyOutboundSide = m.notifyOutboundSide.factory(**inboundCallParams['notifyOutboundSide'])

  inboundCall.set_connect(connect)
  inboundCall.set_callerID(callerID)
  inboundCall.set_notifyOutboundSide(notifyOutboundSide)

  # create outboundCallHandling
  outboundCall = m.outboundCallHandling.factory()
  #   create dial
  #     - DTMFDigitDurationInMs
  #     - interDigitDelayInMs
  #     - waitForDialTone
  #     - preDialDelayInMs
  dial = m.dial.factory(**outboundCallParams['dial'])
  #   create connectSupervision
  connectOutboundSide = m.connectOutboundSide.factory(\
      **outboundCallParams['connectOutboundSide'])
  outboundConnect = m.connect.factory(**outboundCallParams['connect'])
  connectSupervision = m.connectSupervision.factory(
      connectOutboundSide=connectOutboundSide, connect=outboundConnect)

  outboundCall.add_dial(dial)
  outboundCall.set_connectSupervision([connectSupervision])
  
  # create disconnectSupervision
  disconnectSupervision = m.disconnectSupervision.factory()
  #   create disconnectDetectors
  disconnectDetectors = m.disconnectDetectors.factory()

  # add default ringCadenceDetector
  disconnectDetectors.set_ringCadenceDetector(\
      m.ringCadenceDetector.factory(\
        **disconnectSupervisionParams['ringCadenceDetector']))

  #   create each non-cadence detector
  for detectorValues in disconnectSupervisionParams['detectors']:
    disconnectDetectors.add_detector(m.detector.factory(**detectorValues))

  disconnectSupervision.set_disconnectDetectors(disconnectDetectors)

  # create analogFxoConfig
  analogFxo = m.analogFxoConfig.factory(**configParams)
  analogFxo.set_inboundCallHandling(inboundCall)
  analogFxo.set_outboundCallHandling(outboundCall)
  analogFxo.set_disconnectSupervision(disconnectSupervision)
  analogFxo.set_ID(getNextAnalogConfID(pstnConfig))

  # add analogFxoConfig to analog section of callControl
  analog.add_analogFxoConfig(analogFxo)

  return analogFxo
  
###############################################################################
# Add a virtual digital board
# numInterfaces can be of any value, as long as it is an even number
# The first interface will be peered to the last interface, the second to the 
#   one before the last, etc.
# @returns Sangoma Virtual Digital Board (already added to the PSTN Config)
def addSangomaVirtualDigitalBoard(pstnConfig, boardID, numInterfaces):

  # Must be an even number of interfaces
  assert not numInterfaces % 2

  interfaceList = []
  for i in range(1, numInterfaces + 1):
    interfaceList.append(m.interface.factory(
      ID=i, 
      displayName='B%uI%u' % (boardID, i),
      peerVirtualInterfaceID=numInterfaces-i+1))

  sangomaVirtualDigitalBoard = m.sangomaVirtualDigitalBoard.factory(
      ID=boardID, 
      displayName='B%u - virtual' % (boardID),
      model='VIRTUAL_DIGITAL',
      interface=interfaceList) # "interface" here is a list
  
  pstnConfig.get_boards().add_sangomaVirtualDigitalBoard(
      sangomaVirtualDigitalBoard)

  return sangomaVirtualDigitalBoard 


###############################################################################
# Get list of interfaceRefs for which nfas group applies to
def getInterfaceRefsForNfasGroup(nfasGroup):
  members = nfasGroup.get_members()
  # Construct members object if none exists
  if members is None:
    members = m.members.factory()
    nfasGroup.set_members(members)
  return members.get_interfaceRef()

###############################################################################
# Add interface to the members section of a nfas group
def addInterfaceToNfasMembers(nfasGroup, board, interface, nfasInterfaceID,\
    DChannelMode, BChannelRangeList=None):
  # NOTE not checking if board is already there
  getInterfaceRefsForNfasGroup(nfasGroup).append(m.interfaceRef.factory(
    boardID=board.get_ID(),
    interfaceID=interface.get_ID(),
    nfasInterfaceID=nfasInterfaceID,
    DChannelMode=DChannelMode,
    BChannelRange=BChannelRangeList))

###############################################################################
# Get list of interfaceRefs for which config applies to
def getInterfaceRefsForConfig(config):
  applyTo = config.get_applyTo()
  # Construct applyTo object if none exists
  if applyTo is None:
    applyTo = m.applyTo.factory()
    config.set_applyTo(applyTo)
  return applyTo.get_interfaceRef()

###############################################################################
# Apply config to board (not enumerating its interfaces)
def applyConfigToBoard(config, board):
  # NOTE not checking if board is already there
  getInterfaceRefsForConfig(config).append(m.interfaceRef.factory(
    boardID=board.get_ID()))

###############################################################################
# Apply config to interface
def applyConfigToInterface(config, board, interface, BChannelRangeList=None,
    channelGroupID=None):
  # NOTE not checking if interface is already there
  getInterfaceRefsForConfig(config).append(m.interfaceRef.factory(
    boardID=board.get_ID(), 
    interfaceID=interface.get_ID(),
    BChannelRange=BChannelRangeList,
    channelGroupID=channelGroupID))

###############################################################################
# Apply config to all interfaces of a board
def applyConfigToBoardInterfaces(config, board, BChannelRangeList=None,
    channelGroupID=None):
  for interface in board.get_interface():
    applyConfigToInterface(config, board, interface, BChannelRangeList,
        channelGroupID)

###############################################################################
# Add a channel group to the callControl
def addChannelGroup(pstnConfig, **configParams):
  channelGroups = pstnConfig.get_callControl().get_channelGroups()
  if channelGroups is None:
    channelGroups = m.channelGroups.factory()
    pstnConfig.get_callControl().set_channelGroups(channelGroups)

  channelGroup = m.channelGroup.factory(**configParams)
  channelGroups.add_channelGroup(channelGroup)

###############################################################################
# TODO Move together with object registry for editor elsewhere
def getJSONForEditor(in_pstnConfig):
  # Used to sort the lists
  from operator import itemgetter  
  # TreePanel expects a list of Nodes
  jsonTree = []

  if in_pstnConfig is None:    
    return jsonTree

  jsonBoards = []
  # Digital boards
  for board in in_pstnConfig.get_boards().get_sangomaDigitalBoard():
    jsonInterfaces = []
    for interface in board.get_interface():
      jsonInterfaces.append({
        'text' : interface.get_displayName(),
        'id'   : registerObjectForEditor(interface),
        'leaf' : 'true',
        'cls'  : 'file',
        'frm'  : 'frmDigitalInterface'
        })
    jsonBoards.append({
      'text'      : board.get_displayName(),
      'id'        : registerObjectForEditor(board),
      'leaf'      : 'false',
      'cls'       : 'x-tree-node-collapsed',
      'frm'       : 'panelDigitalBoard',
      'children'  : jsonInterfaces
      })

  # Analog boards
  for board in in_pstnConfig.get_boards().get_sangomaAnalogBoard():
    jsonInterfaces = []
    for interface in board.get_interface():
      jsonInterfaces.append({
        'text' : interface.get_displayName(),
        'id'   : registerObjectForEditor(interface),
        'leaf' : 'true',
        'cls'  : 'file',
        'frm'  : 'frmAnalogInterface'
        })
    jsonBoards.append({
      'text'      : board.get_displayName(),
      'id'        : registerObjectForEditor(board),
      'leaf'      : 'false',
      'cls'       : 'x-tree-node-collapsed',
      'frm'       : 'panelAnalogBoard',
      'children'  : jsonInterfaces
      })
    
  # Virtual Digital boards
  for board in in_pstnConfig.get_boards().get_sangomaVirtualDigitalBoard():
    jsonInterfaces = []
    for interface in board.get_interface():
      jsonInterfaces.append({
        'text' : interface.get_displayName(),
        'id'   : registerObjectForEditor(interface),
        'leaf' : 'true',
        'cls'  : 'file',
        'frm'  : 'frmVirtualDigitalInterface'
        })
    jsonBoards.append({
      'text'      : board.get_displayName(),
      'id'        : registerObjectForEditor(board),
      'leaf'      : 'false',
      'cls'       : 'x-tree-node-collapsed',
      'frm'       : 'panelVirtualDigitalBoard',
      'children'  : jsonInterfaces
      })

  jsonBoards.sort()

  # All Boards
  jsonTree.append({
    'text'      : 'Boards',
    'id'        : 'boards', # ID for display only
    'leaf'      : 'false',
    'cls'       : 'x-tree-node-collapsed',
    'frm'       : 'gridBoards',
    'children'  : jsonBoards
    })

  # PhysicalInterfaceConfig
  jsonPhysicalInterfaceConfigs = []
  # Digital
  for sangomaConfig in in_pstnConfig.\
      get_physicalInterfaceConfigs().get_sangomaDigitalConfig():
    displayName = sangomaConfig.get_displayName()
    if sangomaConfig.get_digitalE1() is None: #T1 board
      jsonPhysicalInterfaceConfigs.append({
        'text'  : displayName,
        'id'    : registerObjectForEditor(sangomaConfig),
        'leaf'  : 'true',
        'cls'   : 'file',
        'frm'   : 'frmDigitalT1PhysConf'
      })
    else: #E1 board
      jsonPhysicalInterfaceConfigs.append({
        'text'  : displayName,
        'id'    : registerObjectForEditor(sangomaConfig),
        'leaf'  : 'true',
        'cls'   : 'file',
        'frm'   : 'frmDigitalE1PhysConf'
      })
  # Analog
  for sangomaConfig in in_pstnConfig.\
      get_physicalInterfaceConfigs().get_sangomaAnalogConfig():
    displayName = sangomaConfig.get_displayName()
    jsonPhysicalInterfaceConfigs.append({
      'text'  : displayName,
      'id'    : registerObjectForEditor(sangomaConfig),
      'leaf'  : 'true',
      'cls'   : 'file',
      'frm'   : 'frmAnalogPhysConf'
      })

  jsonPhysicalInterfaceConfigs = sorted(jsonPhysicalInterfaceConfigs,key=itemgetter('id'))

  jsonTree.append({
    'text'      : 'Physical Configurations',
    'id'        : 'physicalConf',   
    'leaf'      : 'false',
    'cls'       : 'x-tree-node-collapsed',
    'frm'       : 'gridPhysicalConfig',
    'children'  : jsonPhysicalInterfaceConfigs
  })

  # CallControl 
  #   Channel Groups
  jsonChannelGroups = []
  for channelGroup in in_pstnConfig.get_callControl().\
      get_channelGroups().get_channelGroup():
    jsonChannelGroups.append({
      'text'  : channelGroup.get_ID(),
      'id'    : registerObjectForEditor(channelGroup),
      'leaf'  : 'true',
      'cls'   : 'file',
      'frm'   : 'frmChannelGroup'
      })

  #   ISDN configuration group
  #     FAS config groups
  jsonISDNConfigs = []
  isdn = in_pstnConfig.get_callControl().get_isdn()
  if isdn:
    for fasConfig in in_pstnConfig.get_callControl().\
        get_isdn().get_fasConfig():
      displayName = fasConfig.get_displayName()
      jsonISDNConfigs.append({
        'text'  : displayName,
        'id'    : registerObjectForEditor(fasConfig),
        'leaf'  : 'true',
        'cls'   : 'file',
        'frm'   : 'frmIsdnFas'
        })

    # NFAS config groups
    for nfasGroup in in_pstnConfig.get_callControl().\
        get_isdn().get_nfasGroup():
      displayName = nfasGroup.get_displayName()
      jsonISDNConfigs.append({
        'text'  : displayName,
        'id'    : registerObjectForEditor(nfasGroup),
        'leaf'  : 'true',
        'cls'   : 'file',
        'frm'   : 'frmIsdnNfas'
        })

  #   Analog configuration group
  jsonAnalogConfigs = []
  analog = in_pstnConfig.get_callControl().get_analog()
  if analog:
    #     Fxo config
    for fxo in in_pstnConfig.get_callControl().\
        get_analog().get_analogFxoConfig():
      displayName = fxo.get_displayName()
      jsonAnalogConfigs.append({
        'text'  : displayName,
        'id'    : registerObjectForEditor(fxo),
        'leaf'  : 'true',
        'cls'   : 'file',
        'frm'   : 'frmAnalogFxo'
        })

    #     Fxs config
    for fxs in in_pstnConfig.get_callControl().\
        get_analog().get_analogFxsConfig():
      displayName = fxs.get_displayName()
      jsonAnalogConfigs.append({
        'text'  : displayName,
        'id'    : registerObjectForEditor(fxs),
        'leaf'  : 'true',
        'cls'   : 'file',
        'frm'   : 'frmAnalogFxs'
        })

  jsonChannelGroups = sorted(jsonChannelGroups,key=itemgetter('id'))
  jsonCallControlNode = {
    'text'      : 'Call Control',
    'id'        : 'callControl', # ID for display only    
    'leaf'      : 'false',
    'cls'       : 'x-tree-node-collapsed',
    'children'  : [{
                    'text'      : 'Channel Groups',
                    'id'        : 'channelGroup', # ID for display only
                    'leaf'      : 'false',
                    'cls'       : 'x-tree-node-collapsed',
                    'frm'       : 'gridChannelGroup',
                    'children'  : jsonChannelGroups
                  }]
    }

  jsonISDNConfigs = sorted(jsonISDNConfigs,key=itemgetter('id'))
  jsonCallControlNode['children'].append({
    'text'      : 'ISDN configurations',
    'id'        : 'isdnConfig', # ID for display only
    'leaf'      : 'false',
    'cls'       : 'x-tree-node-collapsed',
    'frm'       : 'gridISDNConfiguration',
    'children'  : jsonISDNConfigs
    })


  jsonAnalogConfigs = sorted(jsonAnalogConfigs,key=itemgetter('id'))
  jsonCallControlNode['children'].append({
    'text'      : 'Analog configurations',
    'id'        : 'analogConfig', # ID for display only
    'leaf'      : 'false',
    'cls'       : 'x-tree-node-collapsed',
    'frm'       : 'gridAnalogConfiguration',
    'children'  : jsonAnalogConfigs
    })

  jsonTree.append(jsonCallControlNode)

  return jsonTree 


###############################################################################
# Runtests
if __name__ == '__main__':

    def runtest(testName):

      if testName == 'sangomaDefault':
        writeConfigToFileAsXML(createDefaultSangomaPstnConfig(), sys.stdout)

      elif testName == 'oneA104board':
        pstnConfig = createDefaultSangomaPstnConfig()
        addSangomaDigitalBoard(pstnConfig, boardModel='A104', 
            boardID=1, starting_wanpipe=1)
        writeConfigToFileAsXML(pstnConfig, sys.stdout)

      elif testName == 'oneA104boardWithPhysicalAndFasConfig':

        pstnConfig = createDefaultSangomaPstnConfig()

        digitalBoard = addSangomaDigitalBoard(pstnConfig, boardModel='A104', 
            boardID=1, starting_wanpipe=1)

        physicalDigitalConfig = addT1SangomaDigitalConfig(pstnConfig,
            'Digital Config',
            defaultT1Config,
            defaultAudioParams)

        # Indicate physical config applies to all interfaces of board
        applyConfigToBoardInterfaces(physicalDigitalConfig, digitalBoard)

        fasConfig = addFasConfig(pstnConfig, **defaultFasConfig)

        # Indicate FAS config applies to all channels of board
        BChannelRange = m.BChannelRange.factory(firstChannel=1, 
            lastChannel=23, channelGroupID='default')
        applyConfigToBoardInterfaces(fasConfig, digitalBoard, [BChannelRange])

        writeConfigToFileAsXML(pstnConfig, sys.stdout)

      elif testName == 'oneA200boardWithConfig':

        pstnConfig = createDefaultSangomaPstnConfig()

        analogBoard = addSangomaAnalogBoard(pstnConfig, boardModel='A200', 
            boardID=1, wanpipe=1, numInterfaces=4)

        analogPhysicalConfig = addSangomaAnalogConfig(pstnConfig,
          'Analog Config',
          'FCC',
          defaultAudioParams)
        
        # Indicate physical config applies to board (analog physical configs 
        #   are per board)
        applyConfigToBoard(analogPhysicalConfig, analogBoard)

        configParams = {}
        configParams['displayName'] = 'FXO-config1'
        analogFxoConfig = addAnalogFxoConfig(pstnConfig, configParams)

        # Indicate FXO config applies to all interfaces of board
        applyConfigToBoardInterfaces(analogFxoConfig, analogBoard, 
            channelGroupID='default')

        writeConfigToFileAsXML(pstnConfig, sys.stdout)

      elif testName == 'oneVirtualDigitalBoardWithConfig':

        # Must be even
        NUM_INTERFACES = 16

        pstnConfig = createDefaultSangomaPstnConfig()

        virtualDigitalBoard = addSangomaVirtualDigitalBoard(pstnConfig,
            boardID=1, numInterfaces=NUM_INTERFACES)

        # Pretty much all physical config ignored with virtual boards, 
        #   but must still be provided (giving defaults)
        physicalDigitalConfig = addT1SangomaDigitalConfig(pstnConfig,
            'Digital Config',
            defaultT1Config,
            defaultAudioParams)

        # Indicate physical config applies to all interfaces of board
        applyConfigToBoardInterfaces(physicalDigitalConfig, virtualDigitalBoard)

        # Virtual boards must have one set of interfaces using TERMINAL 
        #   termination peered to interfaces using NETWORK termination
        fasConfigParams = defaultFasConfig
        fasConfigParams['displayName'] = 'Terminal'
        fasConfigParams['termination'] = 'TERMINAL'
        fasTerminalConfig = addFasConfig(pstnConfig, **fasConfigParams)

        fasConfigParams['displayName'] = 'Network'
        fasConfigParams['termination'] = 'NETWORK'
        fasNetworkConfig = addFasConfig(pstnConfig, **fasConfigParams)

        BChannelRange = m.BChannelRange.factory(firstChannel=1, 
            lastChannel=23, channelGroupID='default')

        # Apply TERMINAL config to first half of interfaces
        for interface in virtualDigitalBoard.get_interface()[:NUM_INTERFACES/2]:
          applyConfigToInterface(fasTerminalConfig, virtualDigitalBoard,
              interface, [BChannelRange])

        # Apply NETWORK config to second half of interfaces
        for interface in virtualDigitalBoard.get_interface()[NUM_INTERFACES/2:]:
          applyConfigToInterface(fasNetworkConfig, virtualDigitalBoard,
              interface, [BChannelRange])

        writeConfigToFileAsXML(pstnConfig, sys.stdout)

      elif testName == 'getJSONForEditor':

        pstnConfig = m.parse('test-input/pstn-config-digital-only.xml')
        print getJSONForEditor(pstnConfig)

      else:

        print 'Invalid runtest "%s"' % testName


    args = sys.argv[1:]
    if len(args) == 1:
      runtest(args[0])
    else:
      print 'Must pass runtest name as argument'
