import turbogears
from odict import odict
from turbogears import controllers, expose
from turbogears import flash, redirect, url

from paraxip.gateway_gui import extjs
from paraxip.gateway_gui_widgets.tabpstn  import TabPSTN
from paraxip.gateway_gui_widgets.treeviewpstn  import TreeViewPSTN

from pstnconfig import model
from operator import itemgetter
import simplejson
import os

# from pstnconfig import model
import logging
log = logging.getLogger("pstnconfig.controllers")


class ConfigController(controllers.Controller):

  def __init__ (self, in_strProjectPath):
    log.info("Initializing config controller")
    strSrcFile = os.path.join(in_strProjectPath, "pstnconfig/static/"
                                         "javascript/widgets_src/editor.js")
    self.extTree = TreeViewPSTN('pstnTree', renderTo='tree', 
                                fetch='config/fetchTree', 
                                src_file=strSrcFile)
    
    #Elements that will build our tabber
    tabs = odict()
    #Each elements of the tabs dictionnary has the format:
    # tabs['Tab Text'] = ['/autoloadUrl','tabId']
    #tabs['Routing Rules'] = ['/config/routing','routingRulesTabId']
    tabs['Routing Rules'] = ['/static/gateway_down.html','routingRulesTabId']
    tabs['PSTN Config'] = ['','pstnEditorTabId']
    
    items = [{'title': k, 'autoLoad': v[0], 'id': v[1]} for k, v in tabs.items()]
    tabList = [(k.replace(" ","").lower(),v[0],v[1]) for k, v in tabs.items()]
    tabs = dict(tabs)
    
    strSrcFile = os.path.join(in_strProjectPath, "pstnconfig/static/"
                              "javascript/widgets_src/pstn_config_tabs.js")
    
    self.extSubTabs = TabPSTN('configSubTabs',tabs=tabs, items=items, 
                              tabList=tabList, width= 757, height=595,
                              src_file=strSrcFile,children=[self.extTree])


  def getSubTabs (self):
    return self.extSubTabs

  @expose()
  def index (self):
    """Displays the configuration options to the user"""

    return dict(extSubTabs=extSubTabs)

  @expose(template='pstnconfig.templates.config.routing')
  def routing (self):
    """Displays the routing rules"""
    return {}
 
  @expose()
  def editor(self):
    return dict(extTree = self.extTree)

  ##########################################################################
  # Fetch Data provider for the root Ext.tree.AsyncTreeNode in the TreeView
  # Returns a json-encoded string a tree representation of the configuration
  @expose()
  def fetchTree(self, node):
    # Load current pstnconfig model if not done already
    # TODO check for errors
    if model.currentConfig is None and model.PSTNConfigFileExists():      
      model.currentConfig = model.parse(model.getPSTNConfigFileName())              
    return simplejson.dumps(model.getJSONForEditor(model.currentConfig))

  ##########################################################################
  # Returns a dictionary exposing the values of the board corresponding to 
  # the passed id.
  # The fields to be exposed to the user are:
  #   - registered id (hidden field).
  #   - The display name of the board
  #   - The type of the board (digital in that case)
  #   - The model of the board (A101,A102,A104 or A108)
  #   - The number of interface for this board
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmDigitalBoardData(self, id):
    log.info("Getting board data for board id: %s" % id)
    board = model.getObjectFromEditorId(id)
    boardModel = board.get_model() #get the board model
    data = {
      'id': board.get_ID(),
      'nodeId': id,
      'displayName': board.get_displayName(),
      'type': model.boardsModelInfo[boardModel]['type'],
      'model': boardModel,
      'numInterfaces': model.boardsModelInfo[boardModel]['numInterfaces'],
    }             
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a dictionary exposing the values of the board corresponding to 
  # the passed id.
  # The fields to be exposed to the user are:
  #   - registered id (hidden field).
  #   - The display name of the board
  #   - The type of the board (analog in that case)
  #   - The model of the board (A200)
  #   - The number of interfaces
  #   - The wanpipe number
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmAnalogBoardData(self, id):
    log.info("Getting board data for board id: %s" % id)
    board = model.getObjectFromEditorId(id)
    boardModel = board.get_model() #get the board model
    data = {
      'id': board.get_ID(),
      'nodeId': id,
      'displayName': board.get_displayName(),
      'type': model.boardsModelInfo[boardModel]['type'],
      'model': boardModel,
      'numInterfaces': len(board.get_interface()),
      'wanpipe': board.get_wanpipe()
    }             
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a dictionary exposing the values of the virtual digital board 
  #   corresponding to the passed id.
  # The fields to be exposed to the user are:
  #   - registered id (hidden field).
  #   - The display name of the board
  #   - The type of the board (analog in that case)
  #   - The model of the board (A200)
  #   - The number of interfaces
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmVirtualDigitalBoardData(self, id):
    log.info("Getting board data for board id: %s" % id)
    board = model.getObjectFromEditorId(id)
    boardModel = board.get_model() #get the board model
    data = {
      'id': board.get_ID(),
      'nodeId': id,
      'displayName': board.get_displayName(),
      'type': model.boardsModelInfo[boardModel]['type'],
      'model': boardModel,
      'numInterfaces': len(board.get_interface())
    }             
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a list of all the boards, analog and digital, listed in the
  # current configuration.
  # The fields to be exposed to the user are:
  #   - Real ID of the board in the config, not the one generated.
  #   - The display name of the board
  #   - The type of the board (analog in that case)
  #   - The model of the board (A200,A101,A102,etc.)
  #   - The number of interfaces
  @expose() # Ext.form.Action.load - expects JSON
  def getGridBoardsData(self, id):
    data = []
    # Append digital boards data
    for board in model.currentConfig.get_boards().get_sangomaDigitalBoard():
      boardModel = board.get_model() #get the board model
      data.append({
        'id': board.get_ID(),
        'displayName': board.get_displayName(),
        'type': model.boardsModelInfo[boardModel]['type'],
        'model': boardModel,
        'numInterfaces': model.boardsModelInfo[boardModel]['numInterfaces'],
        'nodeId': model.getObjectId(board)
      })
    # Analog boards
    for board in model.currentConfig.get_boards().get_sangomaAnalogBoard():
      boardModel = board.get_model() #get the board model
      data.append({
        'id': board.get_ID(),
        'displayName': board.get_displayName(),
        'type': model.boardsModelInfo[boardModel]['type'],
        'model': boardModel,
        'numInterfaces': len(board.get_interface()),
        'nodeId': model.getObjectId(board)
      })
    # Virtual Digital boards
    for board in model.currentConfig.get_boards().get_sangomaVirtualDigitalBoard():
      boardModel = board.get_model() #get the board model
      data.append({
        'id': board.get_ID(),
        'displayName': board.get_displayName(),
        'type': model.boardsModelInfo[boardModel]['type'],
        'model': boardModel,
        'nodeId': model.getObjectId(board)
      })
    data = sorted(data, key=itemgetter('id'))
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns the data for the interface associated to the given id.
  # The fields to be exposed to the user are:
  #   - registered id (hidden field).
  #   - The display name of the board
  #   - The wanpipe number of this interface
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmDigitalInterfaceData(self, id):
    # Get the instance of the interface
    interface =  model.getObjectFromEditorId(id)
    data = {
      'id': interface.get_ID(),
      'displayName': interface.get_displayName(),
      'wanpipe': interface.get_wanpipe(),
      'nodeId': id
    }             
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns the data for the interface associated to the given id.
  # The fields to be exposed to the user are:
  #   - registered id (hidden field).
  #   - The display name of the board
  #   - The wanpipeInterface number of this interface
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmAnalogInterfaceData(self, id):
    # Get the instance of the interface
    interface =  model.getObjectFromEditorId(id)
    data = {
      'id': interface.get_ID(),
      'displayName': interface.get_displayName(),
      'wanpipeInterface': interface.get_wanpipeInterface(),
      'nodeId': id
    }             
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns the data for the virutal digital interface associated to the 
  #   given id.
  # The fields to be exposed to the user are:
  #   - registered id (hidden field).
  #   - The display name of the board
  #   - The peerVirtualInterfaceID of this interface
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmVirtualDigitalInterfaceData(self, id):
    # Get the instance of the interface
    interface =  model.getObjectFromEditorId(id)
    data = {
      'id': interface.get_ID(),
      'displayName': interface.get_displayName(),
      'peerVirtualInterfaceID': interface.get_peerVirtualInterfaceID(),
      'nodeId': id
    }             
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a list of interface for the digital board's given id.
  # The fields to be exposed to the user are:
  #   - Real ID of the board in the config, not the one generated.
  #   - The display name of the board
  #   - The wanpipe number of this interface
  @expose() # Ext.form.Action.load - expects JSON
  def getGridDigitalInterfacesData(self, id):
    # Get the instance of the board
    board = model.getObjectFromEditorId(id)
    data = []

    for i in board.get_interface():
      data.append({
        'id': i.get_ID(),
        'displayName': i.get_displayName(),
        'wanpipe': i.get_wanpipe(),
        'nodeId': model.getObjectId(i)
      })
    data.sort()
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a list of interface for the analog board's given id.
  # The fields to be exposed to the user are:
  #   - Real ID of the board in the config, not the one generated.
  #   - The display name of the board
  #   - The wanpipeInterface number of this interface
  @expose() # Ext.form.Action.load - expects JSON
  def getGridAnalogInterfacesData(self, id):
    # Get the instance of the board
    board = model.getObjectFromEditorId(id)
    data = []

    for i in board.get_interface():
      data.append({
        'id': i.get_ID(),
        'displayName': i.get_displayName(),
        'wanpipeInterface': i.get_wanpipeInterface(),
        'nodeId': model.getObjectId(i)
      })
    data.sort()
                 
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a list of all the physical configs (E1, T1, Analog), available
  # in the system.
  # The fields to be exposed to the user are:
  #   - Real ID of the board in the config.
  #   - The display name of the configs
  @expose() # Ext.form.Action.load - expects JSON
  def getGridPhysicalConfigData(self, id):
    # Get the physicalConfig for digital boards
    data = []
    for physicalConfig in model.currentConfig.get_physicalInterfaceConfigs().get_sangomaDigitalConfig():
      data.append({
        'id': physicalConfig.get_ID(),
        'displayName': physicalConfig.get_displayName(),
        'nodeId': model.getObjectId(physicalConfig)
      })
    # Get the physicalConfig for analog boards
    for physicalConfig in model.currentConfig.get_physicalInterfaceConfigs().get_sangomaAnalogConfig():
      data.append({
        'id': physicalConfig.get_ID(),
        'displayName': physicalConfig.get_displayName(), 
        'nodeId': model.getObjectId(physicalConfig)
      })
    data = sorted(data, key=itemgetter('id'))
    #data.sort()

    return(str(extjs.FormActionLoadResponse(success=True, data=data)))
    
  ##########################################################################
  # Returns a list of all the values associated to the selected T1 config.
  # The fields to be exposed to the user are:
  #   - registered id (hidden field).
  #   - The display name of the configs
  #   - DigitalT1 values
  #   - DChannelT1 values
  #   - Audio values including EchoCancellation, AcousticEchoCancellation, etc.
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmDigitalT1PhysConfData(self, id):
    config = model.getObjectFromEditorId(id)
    digital = config.get_digitalT1()
    dchannel = digital.get_DChannelT1()
    audio = config.get_audio()
    data = {
      't1NodeId': id,
      'id_textT1': config.get_ID(),
      'name_textT1': config.get_displayName(),
      #DigitalT1 values
      'framingT1_selection': digital.get_framing(),
      'line_encT1_selection': digital.get_lineEncoding(),
      'lboT1_selection': digital.get_LBO(),
      'clockingT1_selection': digital.get_clocking(),
      'loopback_modeT1_selection': digital.get_loopbackMode(),
      #Audio values
      'defaultPcmLawT1_selection': audio.get_defaultPcmLaw(),
      'voicePacketLengthInMsT1_text':audio.get_voicePacketLengthInMs(),
      'toneSetAttributesT1_select':audio.get_toneSetName(),
      #EchoCancellation values
      'ecT1_type_selection':audio.get_EchoCancellation().get_type(),
      'ecT1_mode_selection':audio.get_EchoCancellation().get_mode(),
      'ecT1_comfortNoiseMode_selection':audio.get_EchoCancellation().get_comfortNoiseMode(),
      'ecT1_tailDisplacementInMs_text':audio.get_EchoCancellation().get_tailDisplacementInMs(),
      'ecT1_doubleTalkBehavior_selection':audio.get_EchoCancellation().get_doubleTalkBehavior(),
      #Acoustic values
      'aecT1_type_selection':audio.get_AcousticEchoCancellation().get_type(),
      'aecT1_mode_selection':audio.get_AcousticEchoCancellation().get_mode(),
      #LevelControl values
      'levelT1_type_selection':audio.get_LevelControl().get_type(),
      'levelT1_direction_selection':audio.get_LevelControl().get_direction(),
      'levelT1_mode_selection':audio.get_LevelControl().get_mode(),
      'levelT1_targetLevelIndBM0_text':audio.get_LevelControl().get_targetLevelIndBM0(),
      'levelT1_gainIndB_text':audio.get_LevelControl().get_gainIndB(),
      #BackgroundNoiseAttenuation values
      'bnaT1_type_selection':audio.get_BackgroundNoiseAttenuation().get_type(),
      'bnaT1_direction_selection':audio.get_BackgroundNoiseAttenuation().get_direction(),
      'bnaT1_mode_selection':audio.get_BackgroundNoiseAttenuation().get_mode(),
      'bnaT1_attenuationIndB_selection':audio.get_BackgroundNoiseAttenuation().get_attenuationIndB(),
      #DTMFRemoval values
      'dtmfT1_type_selection':audio.get_DTMFRemoval().get_type(),
      'dtmfT1_direction_selection':audio.get_DTMFRemoval().get_direction(),
      'dtmfT1_mode_selection':audio.get_DTMFRemoval().get_mode()
    }             
    #DChannelT1 values
    if dchannel is None:
      data['disableDChannelT1'] = 'DISABLED'
    else:
      data['disableDChannelT1'] = 'ENABLED'
      data['channelT1_idx_text'] = dchannel.get_channelIndex(),
      data['mtu_sizeT1_text'] = dchannel.get_mtuSize(),
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a list of all the values associated to the selected E1 config.
  # The fields to be exposed to the user are:
  #   - registered id (hidden field).
  #   - The display name of the configs
  #   - DigitalE1 values
  #   - DChannelE1 values
  #   - Audio values including EchoCancellation, AcousticEchoCancellation, etc.
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmDigitalE1PhysConfData(self, id):
    config = model.getObjectFromEditorId(id)
    digital = config.get_digitalE1()
    dchannel = digital.get_DChannelE1()
    audio = config.get_audio()
    data = {
      'e1NodeId': id,
      'id_textE1': config.get_ID(),
      'name_textE1': config.get_displayName(),
      #DigitalE1 values
      'signalingE1_selection': digital.get_E1Signaling(),
      'framingE1_selection': digital.get_framing(),
      'line_encE1_selection': digital.get_lineEncoding(),
      'terminationE1_selection': digital.get_E1Termination(),
      'clockingE1_selection': digital.get_clocking(),
      'loopback_modeE1_selection': digital.get_loopbackMode(),
      #Audio values
      'defaultPcmLawE1_selection': audio.get_defaultPcmLaw(),
      'voicePacketLengthInMsE1_text':audio.get_voicePacketLengthInMs(),
      'toneSetAttributesE1_select':audio.get_toneSetName(),
      #EchoCancellation values
      'ecE1_type_selection':audio.get_EchoCancellation().get_type(),
      'ecE1_mode_selection':audio.get_EchoCancellation().get_mode(),
      'ecE1_comfortNoiseMode_selection':audio.get_EchoCancellation().get_comfortNoiseMode(),
      'ecE1_tailDisplacementInMs_text':audio.get_EchoCancellation().get_tailDisplacementInMs(),
      'ecE1_doubleTalkBehavior_selection':audio.get_EchoCancellation().get_doubleTalkBehavior(),
      #Acoustic values
      'aecE1_type_selection':audio.get_AcousticEchoCancellation().get_type(),
      'aecE1_mode_selection':audio.get_AcousticEchoCancellation().get_mode(),
      #LevelControl values
      'levelE1_type_selection':audio.get_LevelControl().get_type(),
      'levelE1_direction_selection':audio.get_LevelControl().get_direction(),
      'levelE1_mode_selection':audio.get_LevelControl().get_mode(),
      'levelE1_targetLevelIndBM0_text':audio.get_LevelControl().get_targetLevelIndBM0(),
      'levelE1_gainIndB_text':audio.get_LevelControl().get_gainIndB(),
      #BackgroundNoiseAttenuation values
      'bnaE1_type_selection':audio.get_BackgroundNoiseAttenuation().get_type(),
      'bnaE1_direction_selection':audio.get_BackgroundNoiseAttenuation().get_direction(),
      'bnaE1_mode_selection':audio.get_BackgroundNoiseAttenuation().get_mode(),
      'bnaE1_attenuationIndB_selection':audio.get_BackgroundNoiseAttenuation().get_attenuationIndB(),
      #DTMFRemoval values
      'dtmfE1_type_selection':audio.get_DTMFRemoval().get_type(),
      'dtmfE1_direction_selection':audio.get_DTMFRemoval().get_direction(),
      'dtmfE1_mode_selection':audio.get_DTMFRemoval().get_mode()
    }             
    #DChannelE1 values
    if dchannel is None:
      data['disableDChannelE1'] = 'DISABLED'
    else:
      data['disableDChannelE1'] = 'ENABLED'
      data['channelE1_idx_text'] = dchannel.get_channelIndex(),
      data['mtu_sizeE1_text'] = dchannel.get_mtuSize(),
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a list of all the values associated to the selected Analog config.
  # The fields to be exposed to the user are:
  #   - registered id (hidden field).
  #   - The display name of the configs
  #   - Analog values
  #   - Audio values including EchoCancellation, AcousticEchoCancellation, etc.
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmAnalogPhysConfData(self, id):
    config = model.getObjectFromEditorId(id)
    analog = config.get_analog()
    audio = config.get_audio()
    data = {
      'analogNodeId': id,
      'id_textAnalog': config.get_ID(),
      'name_textAnalog': config.get_displayName(),
      #Analog values
      'analogVariant_selection':analog.get_variant(),
      #Audio values
      'defaultPcmLawAnalog_selection':audio.get_defaultPcmLaw(),
      'voicePacketLengthInMsAnalog_text':audio.get_voicePacketLengthInMs(),
      'toneSetAttributesAnalog_select':audio.get_toneSetName(),
      #EchoCancellation values
      'ecAnalog_type_selection':audio.get_EchoCancellation().get_type(),
      'ecAnalog_mode_selection':audio.get_EchoCancellation().get_mode(),
      'ecAnalog_comfortNoiseMode_selection':audio.get_EchoCancellation().get_comfortNoiseMode(),
      'ecAnalog_tailDisplacementInMs_text':audio.get_EchoCancellation().get_tailDisplacementInMs(),
      'ecAnalog_doubleTalkBehavior_selection':audio.get_EchoCancellation().get_doubleTalkBehavior(),
      #Acoustic values
      'aecAnalog_type_selection':audio.get_AcousticEchoCancellation().get_type(),
      'aecAnalog_mode_selection':audio.get_AcousticEchoCancellation().get_mode(),
      #LevelControl values
      'levelAnalog_type_selection':audio.get_LevelControl().get_type(),
      'levelAnalog_direction_selection':audio.get_LevelControl().get_direction(),
      'levelAnalog_mode_selection':audio.get_LevelControl().get_mode(),
      'levelAnalog_targetLevelIndBM0_text':audio.get_LevelControl().get_targetLevelIndBM0(),
      'levelAnalog_gainIndB_text':audio.get_LevelControl().get_gainIndB(),
      #BackgroundNoiseAttenuation values
      'bnaAnalog_type_selection':audio.get_BackgroundNoiseAttenuation().get_type(),
      'bnaAnalog_direction_selection':audio.get_BackgroundNoiseAttenuation().get_direction(),
      'bnaAnalog_mode_selection':audio.get_BackgroundNoiseAttenuation().get_mode(),
      'bnaAnalog_attenuationIndB_selection':audio.get_BackgroundNoiseAttenuation().get_attenuationIndB(),
      #DTMFRemoval values
      'dtmfAnalog_type_selection':audio.get_DTMFRemoval().get_type(),
      'dtmfAnalog_direction_selection':audio.get_DTMFRemoval().get_direction(),
      'dtmfAnalog_mode_selection':audio.get_DTMFRemoval().get_mode()
    }             
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns the data for the interface associated to the given id.
  # The fields to be exposed to the user are:
  #   - registered id (hidden field).
  #   - The display name of the board
  #   - The wanpipeInterface number of this interface
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmChannelGroupData(self, id):
    log.info("Getting channel group data for channel id: %s" % id)
    # Get the instance of the interface
    channelGroup =  model.getObjectFromEditorId(id)
    data = {
      'id':channelGroup.get_ID(),
      'nodeId':id,
      'direction':channelGroup.get_direction(),
      'outboundHuntingScheme':channelGroup.get_outboundHuntingScheme()
    }             
    log.info("Loading Channel group data: %s" % data)
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a list of all the channel group configs available
  # in the system.
  # The fields to be exposed to the user are:
  #   - Real ID of the board in the config.
  #   - The display name of the configs
  @expose() # Ext.form.Action.load - expects JSON
  def getGridChannelGroupData(self, id):
    # Get the physicalConfig for digital boards
    data = []
    for channelGroup in model.currentConfig.get_callControl().\
              get_channelGroups().get_channelGroup():
      data.append({
        'id': channelGroup.get_ID(),
        'direction': channelGroup.get_direction(), 
        'outboundHuntingScheme': channelGroup.get_outboundHuntingScheme(),
        'nodeId':model.getObjectId(channelGroup)
      })     
    data = sorted(data, key=itemgetter('id'))
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a list of all the fas and nfasGroup configs available
  # in the system.
  # The fields to be exposed to the user are:
  #   - Real ID of the board in the config.
  #   - The display name of the configs
  @expose() # Ext.form.Action.load - expects JSON
  def getGridISDNConfigurationData(self, id):
    data = []
    isdn = model.currentConfig.get_callControl().get_isdn()
    if isdn:
      for fas in isdn.get_fasConfig():
        data.append({
          'id':fas.get_ID(),
          'displayName':fas.get_displayName(),
          'nodeId':model.getObjectId(fas)
        })
      for nfas in isdn.get_nfasGroup():
        data.append({
          'id':nfas.get_ID(),
          'displayName':nfas.get_displayName(),
          'nodeId':model.getObjectId(nfas)
        })
    data = sorted(data, key=itemgetter('id'))
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns the value of all the attributes of the fas config.
  # The fields to be exposed to the user are:
  #   - Real ID of the config.
  #   - The display name of the config
  #   - Termination
  #   - Switch Variant
  #   - B-Channel Negotiation
  #   - Tone Set Attributes
  #   - Initiate Restart Procedure
  #   - Inband Progress Tones Generation
  #   - Inband Progress Tones Indicator
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmIsdnFasData(self, id):
    fasConfig = model.getObjectFromEditorId(id)
    data = {
      'idFas_text': fasConfig.get_ID(),
      'nodeId':id,
      'nameFas_text': fasConfig.get_displayName(),
      'terminationFas_select': fasConfig.get_termination(),
      'switchVariantFas_select': fasConfig.get_switchVariant(),
      'BChannelNegotiationFas_select':fasConfig.get_BChannelNegotiation(),
      'initRestartProcFas_select':fasConfig.get_initiateRestartProcedure(),
      'inbandProgTonesGenFas_select':fasConfig.get_inbandProgressTonesGeneration(),
      'inbandProgTonesIndFas_select':fasConfig.get_inbandProgressTonesIndicator()
    }                                  

    return(str(extjs.FormActionLoadResponse(success=True, data=data)))
    
  ##########################################################################
  # Returns the value of all the attributes of the nfas config.
  # The fields to be exposed to the user are:
  #   - Real ID of the config.
  #   - The display name of the config
  #   - Termination
  #   - Switch Variant
  #   - B-Channel Negotiation
  #   - Tone Set Attributes
  #   - Initiate Restart Procedure
  #   - Inband Progress Tones Generation
  #   - Inband Progress Tones Indicator
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmIsdnNfasData(self, id):
    nfasGroup = model.getObjectFromEditorId(id)
    data = {
      'idNfas_text': nfasGroup.get_ID(),
      'nodeId':id,
      'nameNfas_text': nfasGroup.get_displayName(),
      'terminationNfas_select': nfasGroup.get_termination(),
      'switchVariantNfas_select': nfasGroup.get_switchVariant(),
      'BChannelNegotiationNfas_select':nfasGroup.get_BChannelNegotiation(),
      'initRestartProcNfas_select':nfasGroup.get_initiateRestartProcedure(),
      'inbandProgTonesGenNfas_select':nfasGroup.get_inbandProgressTonesGeneration(),
      'inbandProgTonesIndNfas_select':nfasGroup.get_inbandProgressTonesIndicator()
    }                                  

    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a list of all the analogFxo and analogFxs configs available
  # in the system.
  # The fields to be exposed to the user are:
  #   - Real ID of the board in the config.
  #   - The display name of the configs
  @expose() # Ext.form.Action.load - expects JSON
  def getGridAnalogConfigurationData(self, id):
    data = []
    analog = model.currentConfig.get_callControl().get_analog()
    if analog:
      for fxo in analog.get_analogFxoConfig():
        data.append({
          'id':fxo.get_ID(),
          'displayName':fxo.get_displayName(),
          'nodeId':model.getObjectId(fxo)
        })
      for fxs in analog.get_analogFxsConfig():
        data.append({
          'id':fxs.get_ID(),
          'displayName':fxs.get_displayName(),
          'nodeId':model.getObjectId(fxs)
        })
    data = sorted(data, key=itemgetter('id'))
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns the value of all the attributes of the analog fxo config.
  # The fields to be exposed to the user are:
  #   - Real ID of the config.
  #   - The display name of the config
  #   - Tone Set Name
  #   - Inbound Call Handling
  #   - Outbound Call Handling
  #   - Connect Supervision
  #   - Disconnect Supervision
  @expose() # Ext.form.Action.load - expects JSON
  def getFrmAnalogFxoData(self, id):
    analogFxo = model.getObjectFromEditorId(id)
    notify =  analogFxo.get_inboundCallHandling().get_notifyOutboundSide()
    inbound = analogFxo.get_inboundCallHandling()
    out_dial = analogFxo.get_outboundCallHandling().get_dial()[0]
    con_super = analogFxo.get_outboundCallHandling().\
                get_connectSupervision()[0]
    ring_cadence = analogFxo.get_disconnectSupervision().\
                   get_disconnectDetectors().get_ringCadenceDetector()
    
    data = {
      'idFxo_text':analogFxo.get_ID(),
      'nodeId':id,
      'nameFxo_text':analogFxo.get_displayName(),
      
      'notifyOutbound_condition_select':notify.get_condition(),
      'numOfRings_text':notify.get_numberOfRings(),
      'notifyOutbound_delay_text':notify.get_delayInMs(),

      'notifyOutboundConnect_condition_select': \
        inbound.get_connect().get_condition(),
      'callerID_type_selection':inbound.get_callerID().get_type(),
      
      'dial_active_selection':out_dial.get_active(),
      'dial_waitForDialTone_selection': out_dial.get_waitForDialTone(),
      'preDialDelayInMs_text': out_dial.get_preDialDelayInMs(),
      'interDigitDelayInMs_text': out_dial.get_interDigitDelayInMs(),
      'dTMFDigitDurationInMs_text': out_dial.get_DTMFDigitDurationInMs(),

      'connectOutBoundSide_condition_selection': \
        con_super.get_connectOutboundSide().get_condition(),
      'connectParamOutbound_condition_selection': \
        con_super.get_connect().get_condition(),
      'connectParamOutbound_delayInMs_text': \
        con_super.get_connect().get_delayInMs(),

      'ringCadenceDetector_active_selection': \
        ring_cadence.get_active(),
      'ringCadenceDetector_OffDelayMs_text': \
        ring_cadence.get_ringCadenceOffDelayMs()

    }                                  
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))
    
  # TODO
  # def getFrmFxsData

#################################
# ApplyTo PhysicalConfigs       #
#################################

  ##########################################################################
  # Returns a list of all the analog boards that are not assigned to a 
  # physical configuration.
  # The fields to be exposed to the user are:
  #   - ID of the board.
  #   - The display name of the board
  @expose() # Ext.form.Action.load - expects JSON
  def getUnassignedBoards(self, id):
    log.info("Getting all the unassigned analog boards")
    isBoardAssigned = False
    data = []
    for board in model.currentConfig.get_boards().get_sangomaAnalogBoard():
      for config in model.currentConfig.get_physicalInterfaceConfigs().get_sangomaAnalogConfig():
        applyTo = config.get_applyTo()
        if applyTo != None:
          for interfaceRef in applyTo.get_interfaceRef():
            if interfaceRef.get_boardID() == board.get_ID():
              isBoardAssigned = True
              break
      if not(isBoardAssigned):
        data.append({
          'id':board.get_ID(),
          'displayName':board.get_displayName(),
          'configID':id
        })
      isBoardAssigned = False
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a list of all the digital interfaces that are not assigned to a 
  # physical configuration.
  # The fields to be exposed to the user are:
  #   - ID of the board.
  #   - ID of the interface
  #   - The display name of the board
  #   - The display name of the interface
  @expose() # Ext.form.Action.load - expects JSON
  def getUnassignedInterfaces(self, id):
    log.info("Getting all the unassigned digital interfaces")
    interfaceRefList = []
    data = []

    for config in model.currentConfig.get_physicalInterfaceConfigs().get_sangomaDigitalConfig():
      applyTo = config.get_applyTo()
      if applyTo != None:
        for interfaceRef in applyTo.get_interfaceRef():
          interfaceRefList.append((interfaceRef.get_boardID(),interfaceRef.get_interfaceID()))

    for board in model.currentConfig.get_boards().get_sangomaDigitalBoard():
      for interface in board.get_interface():
        if (board.get_ID(),interface.get_ID()) not in interfaceRefList:
          data.append({
            'id':interface.get_ID(),
            'boardID':board.get_ID(),
            'displayName':board.get_displayName(),
            'ifName':interface.get_displayName(),            
            'configID':id
          })
    for board in model.currentConfig.get_boards().get_sangomaVirtualDigitalBoard():
      for interface in board.get_interface():
        if (board.get_ID(),interface.get_ID()) not in interfaceRefList:
          data.append({
            'id':interface.get_ID(),
            'boardID':board.get_ID(),
            'displayName':board.get_displayName(),
            'ifName':interface.get_displayName(),
            'configID':id
          })
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns a list of all the analog boards that are assigned to the
  # physical configuration.
  # The fields to be exposed to the user are:
  #   - ID of the board.
  #   - The display name of the board
  @expose() # Ext.form.Action.load - expects JSON
  def getAssignedBoards(self, id):
    log.info("Getting all the analog boards assigned to the config %s" % id)
    data = []
    for config in model.currentConfig.get_physicalInterfaceConfigs().get_sangomaAnalogConfig():
      if config.get_ID() == int(id):
        applyTo = config.get_applyTo()
        if applyTo != None:
          for interfaceRef in applyTo.get_interfaceRef():
            for board in model.currentConfig.get_boards().get_sangomaAnalogBoard():
              if board.get_ID() == interfaceRef.get_boardID():
                data.append({
                  'id':board.get_ID(),
                  'displayName':board.get_displayName(),
                  'configID':id
                })
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))


  ##########################################################################
  # Returns a list of all the digital interfaces that are assigned to the  
  # physical configuration.
  # The fields to be exposed to the user are:
  #   - ID of the board.
  #   - ID of the interface
  #   - The display name of the board
  #   - The display name of the interface
  @expose() # Ext.form.Action.load - expects JSON
  def getAssignedInterfaces(self, id):
    log.info("Getting all the digital interfaces assigned to the config %s" % id)
    data = []
    for config in model.currentConfig.get_physicalInterfaceConfigs().get_sangomaDigitalConfig():
      if config.get_ID() == int(id):
        applyTo = config.get_applyTo()
        if applyTo != None:
          for interfaceRef in applyTo.get_interfaceRef():
            board = self.getBoardObject(int(interfaceRef.get_boardID()))
            interface = self.getInterfaceObject(int(interfaceRef.get_boardID()),int(interfaceRef.get_interfaceID()))
            data.append({
              'id':interfaceRef.get_interfaceID(),
              'boardID':interfaceRef.get_boardID(),
              'displayName':board.get_displayName(),
              'ifName':interface.get_displayName(),
              'configID':id
            })
    return(str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Apply the boards to the given configuration.
  @expose() # Ext.form.Action.load - expects JSON
  def applyBoardsToConfig(self, **data):
    log.info("Apply the boards to the config: %s" % data)
    boardsData = simplejson.loads(data['gridData'])
    physConf = None
    board = None
    #Find the config object
    for config in model.currentConfig.get_physicalInterfaceConfigs().get_sangomaAnalogConfig():
      if config.get_ID() == int( simplejson.loads(data['configID']) ):
        physConf = config
    physConf.set_applyTo(None) #Clear the applyTo section

    if physConf != None:
      for boardInfo in boardsData:
        #Find the board object
        for aBoard in model.currentConfig.get_boards().get_sangomaAnalogBoard():
          if aBoard.get_ID() == int(boardInfo['id']):
            board = aBoard
        if board != None:
          model.applyConfigToBoard(physConf,board)
        board = None

    model.writeCurrentConfigToFile()
    return(str(extjs.FormActionResponse(success=True)))

  ##########################################################################
  # Apply the interfaces to the given configuration.
  @expose() # Ext.form.Action.load - expects JSON
  def applyInterfacesToConfig(self, **data):
    log.info("Apply the interfaces to the config: %s" % data)
    interfacesData = simplejson.loads(data['gridData'])
    physConf = None
    interface = None
    board = None
    #Find the config object
    for config in model.currentConfig.get_physicalInterfaceConfigs().get_sangomaDigitalConfig():
      if config.get_ID() == int( simplejson.loads(data['configID']) ):
        physConf = config
        physConf.set_applyTo(None) #Clear the applyTo section

    if physConf != None:
      for interfaceInfo in interfacesData:
        #Find the board object
        for b in model.currentConfig.get_boards().get_sangomaDigitalBoard():
          if b.get_ID() == int(interfaceInfo['boardID']):
            board = b
            for i in board.get_interface():
              if i.get_ID() == int(interfaceInfo['id']):
                interface = i
        for b in model.currentConfig.get_boards().get_sangomaVirtualDigitalBoard():
          if b.get_ID() == int(interfaceInfo['boardID']):
            board = b
            for i in board.get_interface():
              if i.get_ID() == int(interfaceInfo['id']):
                interface = i
        if interface != None:
          model.applyConfigToInterface(physConf,board,interface)
        interface = None

    model.writeCurrentConfigToFile()
    return(str(extjs.FormActionResponse(success=True)))

#################################
# ApplyTo FasConfigs            #
#################################
  ##########################################################################
  # Returns all the digital boards.
  @expose()
  def getBoards(self,id):
    data = []
    for board in model.currentConfig.get_boards().get_sangomaDigitalBoard():
      data.append({
        'id': board.get_ID(),
        'displayName':board.get_displayName(),
        'nodeId': model.getObjectId(board)
      })
    for board in model.currentConfig.get_boards().get_sangomaVirtualDigitalBoard():
      data.append({
        'id': board.get_ID(),
        'displayName':board.get_displayName(),
        'nodeId': model.getObjectId(board)
      })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns all the digital interfaces of the board corresponding to the 
  # given id.
  @expose()
  def getInterfaces(self,id):
    data = []
    board = self.getBoardObject(int(id))

    for interface in board.get_interface():
      data.append({
        'id': interface.get_ID(),
        'displayName': interface.get_displayName()
      })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns the interface  corresponding to the 
  # given board and interface  id.
  # boardID is expected to be an integer
  # interfaceID is expected to be an integer
  def getInterfaceObject(self,boardID,interfaceID):
    log.info("Get the interface for board: %s interface: %s" % (boardID,interfaceID))
    board = self.getBoardObject(int(boardID))

    for interface in board.get_interface():
      if int(interface.get_ID()) == interfaceID:
        return interface

  ##########################################################################
  # Returns all the channel groups.
  @expose()
  def getChannelGroups(self,id):
    data = []
    for channelGroup in model.currentConfig.get_callControl().get_channelGroups().get_channelGroup():
      data.append({
        'id': channelGroup.get_ID()
      })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns the data in the fas config applyTo section.
  # Returned values are:
  #   - Board id
  #   - Interface id
  #   - First Channel
  #   - Last Channel
  #   - Channel Group ID
  @expose()
  def getFasBChannels(self, id):
    data = []
    fasConfig = None
    for fas in model.currentConfig.get_callControl().get_isdn().get_fasConfig():
      if fas.get_ID() == int(id):
        fasConfig = fas

    if fasConfig:
      applyTo = fasConfig.get_applyTo()
      if applyTo:
        for interfaceRef in applyTo.get_interfaceRef():
          for bChannelRange in interfaceRef.get_BChannelRange():
            data.append({
              'boardId':interfaceRef.get_boardID(),
              'interfaceId':interfaceRef.get_interfaceID(),
              'firstChannel':bChannelRange.get_firstChannel(),
              'lastChannel':bChannelRange.get_lastChannel(),
              'channelGroupID':bChannelRange.get_channelGroupID(),
              'nodeId': model.getObjectId(self.getBoardObject(interfaceRef.get_boardID()))
            })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))

  # Returns the instance of the board matching the given id.
  def getBoardObject(self,boardId):
    for b in model.currentConfig.get_boards().get_sangomaAnalogBoard():
      if b.get_ID() == boardId:
        return b
    for b in model.currentConfig.get_boards().get_sangomaDigitalBoard():
      if b.get_ID() == boardId:
        return b
    for b in model.currentConfig.get_boards().get_sangomaVirtualDigitalBoard():
      if b.get_ID() == boardId:
        return b
    return None

  ##########################################################################
  # Apply the BChannel to the given fas configuration.
  @expose() # Ext.form.Action.load - expects JSON
  def applyBChannelToFasConfig(self,**data):
    log.info("Applying BChannels to fas config: %s" % data)
    id = simplejson.loads(data['fasConfigID'])
    gridData = simplejson.loads(data['gridData'])

    interface = None
    board = None
    fasConfig = None
    for fas in model.currentConfig.get_callControl().get_isdn().get_fasConfig():
      if fas.get_ID() == int(id):
        fasConfig = fas
        fasConfig.set_applyTo(None) #Clear the applyTo section

    if fasConfig :
      for interfaceInfo in gridData:
        #Find the board object
        for b in model.currentConfig.get_boards().get_sangomaDigitalBoard():
          if b.get_ID() == int(interfaceInfo['boardId']):
            board = b
            for i in board.get_interface():
              if i.get_ID() == int(interfaceInfo['interfaceId']):
                interface = i
        for b in model.currentConfig.get_boards().get_sangomaVirtualDigitalBoard():
          if b.get_ID() == int(interfaceInfo['boardId']):
            board = b
            for i in board.get_interface():
              if i.get_ID() == int(interfaceInfo['interfaceId']):
                interface = i
        if interface != None:
          BChannelRange = model.BChannelRange.factory(firstChannel=interfaceInfo['firstChannel'], 
            lastChannel=interfaceInfo['lastChannel'], channelGroupID=interfaceInfo['channelGroupID'])
          model.applyConfigToInterface(fasConfig,board,interface,[BChannelRange])
        interface = None

    model.writeCurrentConfigToFile()
    return(str(extjs.FormActionResponse(success=True)))

#################################
# ApplyTo Nfas Group            #
#################################
  ##########################################################################
  # Returns the data in the nfas group members section.
  # Returned values are:
  #   - Board id
  #   - Interface id
  #   - Nfas Interface Id
  #   - DChannel Mode
  #   - First Channel
  #   - Last Channel
  #   - Channel Group ID
  @expose()
  def getNfasBChannels(self, id):
    data = []
    nfasGroup = None
    for nfas in model.currentConfig.get_callControl().get_isdn().get_nfasGroup():
      if nfas.get_ID() == int(id):
        nfasGroup = nfas

    if nfasGroup:
      members = nfasGroup.get_members()
      if members:
        for interfaceRef in members.get_interfaceRef():
          for bChannelRange in interfaceRef.get_BChannelRange():
            data.append({
              'boardId':interfaceRef.get_boardID(),
              'interfaceId':interfaceRef.get_interfaceID(),
              'nfasInterfaceID':interfaceRef.get_nfasInterfaceID(),
              'DChannelMode':interfaceRef.get_DChannelMode(),
              'firstChannel':bChannelRange.get_firstChannel(),
              'lastChannel':bChannelRange.get_lastChannel(),
              'channelGroupID':bChannelRange.get_channelGroupID(),
              'nodeId': model.getObjectId(self.getBoardObject(interfaceRef.get_boardID()))
            })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns all the digital boards.
  @expose()
  def applyBChannelToNfasConfig(self,**data):
    log.info("Applying BChannels to nfas config: %s" % data)
    id = simplejson.loads(data['nfasGroupID'])
    gridData = simplejson.loads(data['gridData'])

    interface = None
    board = None
    nfasGroup = None
    for nfas in model.currentConfig.get_callControl().get_isdn().get_nfasGroup():
      if nfas.get_ID() == int(id):
        nfasGroup = nfas
        nfasGroup.set_members(None)#Clear the members section

    if nfasGroup :
      for interfaceInfo in gridData:
        #Find the board object
        for b in model.currentConfig.get_boards().get_sangomaDigitalBoard():
          if b.get_ID() == int(interfaceInfo['boardId']):
            board = b
            for i in board.get_interface():
              if i.get_ID() == int(interfaceInfo['interfaceId']):
                interface = i
        for b in model.currentConfig.get_boards().get_sangomaVirtualDigitalBoard():
          if b.get_ID() == int(interfaceInfo['boardId']):
            board = b
            for i in board.get_interface():
              if i.get_ID() == int(interfaceInfo['interfaceId']):
                interface = i
        if interface != None:
          BChannelRange = model.BChannelRange.factory(firstChannel=interfaceInfo['firstChannel'], 
            lastChannel=interfaceInfo['lastChannel'], channelGroupID=interfaceInfo['channelGroupID'])
          model.addInterfaceToNfasMembers(nfasGroup,board,interface,
              interfaceInfo['nfasInterfaceID'],interfaceInfo['DChannelMode'],[BChannelRange])
        interface = None

    model.writeCurrentConfigToFile()
    return(str(extjs.FormActionResponse(success=True)))

#################################
# ApplyTo AnalogFxoConfig       #
#################################
  ##########################################################################
  # Returns all the analog boards.
  @expose()
  def getAnalogBoards(self, id):
    data = []
    for board in model.currentConfig.get_boards().get_sangomaAnalogBoard():
      data.append({
        'id': board.get_ID(),
        'displayName':board.get_displayName(),
        'nodeId': model.getObjectId(board)
      })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns all the analog interfaces of the board corresponding to the 
  # given id.
  @expose()
  def getAnalogInterfaces(self, id):
    data = []
    board = self.getBoardObject(int(id))

    for interface in board.get_interface():
      data.append({
        'id': interface.get_ID(),
        'displayName': interface.get_displayName()
      })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))

  ##########################################################################
  # Returns the data in the fxo config applyTo section.
  # Returned values are:
  #   - Board id
  #   - Interface id
  #   - Channel Group ID
  @expose()
  def getFxoInterfaces(self, id):
    data = []
    fxoConfig = None
    for fxo in model.currentConfig.get_callControl().get_analog().get_analogFxoConfig():
      if fxo.get_ID() == int(id):
        fxoConfig = fxo

    if fxoConfig:
      applyTo = fxoConfig.get_applyTo()
      if applyTo:
        for interfaceRef in applyTo.get_interfaceRef():
          data.append({
            'boardId':interfaceRef.get_boardID(),
            'interfaceId':interfaceRef.get_interfaceID(),
            'channelGroupID':interfaceRef.get_channelGroupID(),
            'nodeId': model.getObjectId(self.getBoardObject(interfaceRef.get_boardID()))
          })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))
  
  ##########################################################################
  # Apply the Interfaces to the given fxo configuration.
  @expose() # Ext.form.Action.load - expects JSON
  def applyInterfacesToFxoConfig(self, **data):
    log.info("Applying Interfaces to fxo config: %s" % data)
    id = simplejson.loads(data['fxoConfigID'])
    gridData = simplejson.loads(data['gridData'])

    interface = None
    board = None
    fxoConfig = None
    for fxo in model.currentConfig.get_callControl().get_analog().get_analogFxoConfig():
      if fxo.get_ID() == int(id):
        fxoConfig = fxo
        fxoConfig.set_applyTo(None) #Clear the applyTo section

    if fxoConfig :
      for interfaceInfo in gridData:
        #Find the board object
        for b in model.currentConfig.get_boards().get_sangomaAnalogBoard():
          if b.get_ID() == int(interfaceInfo['boardId']):
            board = b
            for i in board.get_interface():
              if i.get_ID() == int(interfaceInfo['interfaceId']):
                interface = i
        if interface != None:
          model.applyConfigToInterface(fxoConfig,board,interface,channelGroupID=interfaceInfo['channelGroupID'])
        interface = None

    model.writeCurrentConfigToFile()
    return(str(extjs.FormActionResponse(success=True)))

  ##########################################################################
  # Returns the data in the fxo detector section.
  # Returned values are:
  #   - name
  #   - sequence
  @expose()
  def getFxoDetectors(self, id):
    data = []
    fxoConfig = None
    #
    for fxo in model.currentConfig.get_callControl().get_analog().\
      get_analogFxoConfig():
      if fxo.get_ID() == int(id):
        fxoConfig = fxo
  
    if fxoConfig:
      for detectorRef in fxoConfig.get_disconnectSupervision().\
        get_disconnectDetectors().get_detector():
        data.append({
          'name':detectorRef.get_name(),
          'sequence':detectorRef.get_sequence()
        })

    return (str(extjs.FormActionLoadResponse(success=True, data=data)))

 ##########################################################################
  # Apply the Detectors to the given fxo configuration.
  @expose() # Ext.form.Action.load - expects JSON
  def applyDetectorsToFxoConfig(self, **data):
    log.info("Applying Detectors to fxo config: %s" % data)
    id = simplejson.loads(data['fxoConfigID'])
    gridData = simplejson.loads(data['gridData'])

    detector = None
    board = None
    fxoConfig = None
    for fxo in model.currentConfig.get_callControl().get_analog().get_analogFxoConfig():
      if fxo.get_ID() == int(id):
        fxoConfig = fxo
        fxoConfig.get_disconnectSupervision().\
        get_disconnectDetectors().set_detector([]) #Clear the detector section

    if fxoConfig :
      for detectorInfo in gridData:
        d = model.detector(detectorInfo['name'],detectorInfo['sequence'])
        fxoConfig.get_disconnectSupervision().get_disconnectDetectors().\
            add_detector(d)

    model.writeCurrentConfigToFile()
    return(str(extjs.FormActionResponse(success=True)))


#################################
# Save methods                  #
#################################
  ##########################################################################
  # Saves the modified informations of the board.
  # Editable values are:
  #   - displayName
  @expose()
  def saveBoards (self, **data):
    """Saves the modifications to a board entered by the user in the editor"""

    log.info("Saving board informations with data: %s" % data)

    #Decode received data
    boardData = simplejson.loads(data['boardInfo'])

    #Retrieve the corresponding instance for the board id
    board = model.getObjectFromEditorId(boardData['nodeId'])

    #Reassign modified values    
    boardModel = board.get_model()
    assert model.boardsModelInfo.has_key(boardModel)
    boardModelInfo = model.boardsModelInfo[boardModel]
    if boardModelInfo['type'] == 'analog':
      log.debug('Setting wanpipe number on analog board')
      board.set_wanpipe(boardData['wanpipe'])

    board.set_displayName(boardData['displayName'])


    #save the config afterward
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # Saves the modified informations of the interface.
  # Editable values are:
  #   - displayName
  @expose()
  def saveInterfaces (self, **data):
    """Saves the modifications to an interface entered by the user in the editor"""

    log.info("Saving interface informations with data: %s" % data)

    #Decode received data
    interfaceData = simplejson.loads(data['interfaceInfo'])

    #Retrieve the corresponding instance for the board id
    interface = model.getObjectFromEditorId(interfaceData['nodeId'])

    #Reassign modified values
    #TODO add the other editable fields values (only displayName for now)
    interface.set_displayName(interfaceData['displayName'])

    if interfaceData.has_key('peerVirtualInterfaceID'):
      #Virtual digital interface
      interface.set_peerVirtualInterfaceID(
          interfaceData['peerVirtualInterfaceID'])

    #save the config afterward
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # Saves the new informations of the config.
  # All values are editable except the id.
  @expose()
  def savePhysicalConf (self, **data):
    log.info("Saving physical configuration: %s" % data)
    physConfData = simplejson.loads(data['physicalConfig'])
  
    config = model.getObjectFromEditorId(physConfData['config']['nodeId'])
    #config = None
    #for physConf in model.currentConfig.get_physicalInterfaceConfigs().get_sangomaAnalogConfig():
    #  if physConf.get_ID() == int(physConfData['config']['id']):
    #    config = physConf
    #
    #if config == None:
    #  for physConf in model.currentConfig.get_physicalInterfaceConfigs().get_sangomaDigitalConfig():
    #    if physConf.get_ID() == int(physConfData['config']['id']):
    #      config = physConf

    audio = config.get_audio()

    # Sections are: config, analog, audio, EchoCancellation, etc.
    # Values are the values of that section wich is a dictionary
    for section, values in physConfData.items():
      if section == 'config':
        config.set_displayName(values['displayName'])
      elif section == 'EchoCancellation':
        echoCancel = audio.get_EchoCancellation()
        for attribute, value in values.items():
          attributeMethod = getattr(echoCancel,"set_%s" % attribute)
          attributeMethod(value)
      elif section == 'AcousticEchoCancellation':
        AcousticEchoCancellation = audio.get_AcousticEchoCancellation()
        for attribute, value in values.items():
          attributeMethod = getattr(AcousticEchoCancellation,"set_%s" % attribute)
          attributeMethod(value)
      elif section == 'LevelControl':
        LevelControl = audio.get_LevelControl()
        for attribute, value in values.items():
          attributeMethod = getattr(LevelControl,"set_%s" % attribute)
          attributeMethod(value)
      elif section == 'BackgroundNoiseAttenuation':
        BackgroundNoiseAttenuation = audio.get_BackgroundNoiseAttenuation()
        for attribute, value in values.items():
          attributeMethod = getattr(BackgroundNoiseAttenuation,"set_%s" % attribute)
          attributeMethod(value)
      elif section == 'DTMFRemoval':
        DTMFRemoval = audio.get_DTMFRemoval()
        for attribute, value in values.items():
          attributeMethod = getattr(DTMFRemoval,"set_%s" % attribute)
          attributeMethod(value)
      elif section == 'DChannelT1':
        if values['disableDChannel'] == 'DISABLED':
          config.get_digitalT1().set_DChannelT1(None)
        else:
          dchannelT1 = config.get_digitalT1().get_DChannelT1()
          if dchannelT1 is None:
            dchannelT1 = model.DChannelT1.factory()
          dchannelT1.set_channelIndex(24)
          dchannelT1.set_mtuSize(values['mtuSize'])
          config.get_digitalT1().set_DChannelT1(dchannelT1)
        #for attribute, value in values.items():
        #  attributeMethod = getattr(config.get_digitalT1().get_DChannelT1(),"set_%s" % attribute)
        #  attributeMethod(value)
      elif section == 'DChannelE1':
        if values['disableDChannel'] == 'DISABLED':
          config.get_digitalE1().set_DChannelE1(None)
        else:
          dchannelE1 = config.get_digitalE1().get_DChannelE1()
          if dchannelE1 is None:
            dchannelE1 = model.DChannelE1.factory()
          dchannelE1.set_channelIndex(16)
          dchannelE1.set_mtuSize(values['mtuSize'])
          config.get_digitalE1().set_DChannelE1(dchannelE1)
        #for attribute, value in values.items():
        #  attributeMethod = getattr(config.get_digitalE1().get_DChannelE1(),"set_%s" % attribute)
        #  attributeMethod(value)
      else:
        # Get config attribute method
        method = getattr(config, "get_%s" % section)
        for attribute, value in values.items():
          attributeMethod = getattr(method(),"set_%s" % attribute)
          attributeMethod(value)
  
  
    # Write current config to file using default file location
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # Saves the new informations of the channel group.
  # All values are editable.
  @expose()
  def saveChannelGroup (self, **data):
    log.info("Saving channel group: %s" % data)
    channelData = simplejson.loads(data['channelGroup'])
    channelGroup = model.getObjectFromEditorId(channelData['nodeId'])

    if channelGroup:
      for attribute, value in channelData.items():
        if attribute != 'nodeId':
          method = getattr(channelGroup,"set_%s" % attribute)
          method(value)

    # Write current config to file using default file location
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # Saves the new informations of the config.
  # All values are editable.
  @expose()
  def saveFasConfig (self, **data):
    #log.info("Saving fas config: %s" % data)
    fasData = simplejson.loads(data['fasConfig'])
    fasConfig = model.getObjectFromEditorId(fasData['nodeId'])

    if fasConfig:
      for attribute, value in fasData.items():
        if attribute != 'nodeId':
          method = getattr(fasConfig,"set_%s" % attribute)
          method(value)

    # Write current config to file using default file location
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # Saves the new informations of the config.
  # All values are editable.
  @expose()
  def saveNfasConfig (self, **data):
    log.info("Saving nfas group config: %s" % data)
    nfasData = simplejson.loads(data['nfasGroup'])
    nfasConfig = model.getObjectFromEditorId(nfasData['nodeId'])

    if nfasConfig:
      for attribute, value in nfasData.items():
        if attribute != 'nodeId':
          method = getattr(nfasConfig,"set_%s" % attribute)
          method(value)

    # Write current config to file using default file location
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # Saves the new informations of the config.
  # All values are editable.
  @expose()
  def saveFxoConfig (self, **data):
    log.info("Saving fxo group config: %s" % data)
    fxoData = simplejson.loads(data['frmAnalogFxo'])
    log.info("Saving fxo Data config: %s" % fxoData)
    fxoConfig = model.getObjectFromEditorId(data['nodeId'])

    notify =  fxoConfig.get_inboundCallHandling().get_notifyOutboundSide()
    inbound = fxoConfig.get_inboundCallHandling()
    out_dial = fxoConfig.get_outboundCallHandling().get_dial()[0]
    con_super = fxoConfig.get_outboundCallHandling().\
                get_connectSupervision()[0]
    ring_cadence = fxoConfig.get_disconnectSupervision().\
                   get_disconnectDetectors().get_ringCadenceDetector()
    

    # Sections are: config, analog, audio, EchoCancellation, etc.
    # Values are the values of that section wich is a dictionary
    for section, values in fxoData.items():
      if section == 'fxoConfig':
        fxoConfig.set_displayName(values['displayName'])
      elif section == 'inboundCall':
        notify.set_condition(values['notifyOutbound_condition'])
        if values['notifyOutbound_condition'] == 'AFTER-N-RING':
          notify.set_numberOfRings(values['numberOfRings'])
        else:
          notify.set_numberOfRings(None)
        notify.set_delayInMs(values['notifyOutbound_delay'])
        inbound.get_connect().set_condition(values['connect_condition'])
        inbound.get_callerID().set_type(values['callerID_type'])
      elif section == 'outboundCallDialing':
        out_dial.set_active(values['dial_active'])
        out_dial.set_waitForDialTone(values['dial_waitForDialTone'])
        out_dial.set_preDialDelayInMs(values['preDialDelayInMs'])
        out_dial.set_interDigitDelayInMs(values['interDigitDelayInMs'])
        out_dial.set_DTMFDigitDurationInMs(values['dTMFDigitDurationInMs'])
      elif section == 'outboundCallConnect':
        con_super.get_connectOutboundSide().\
            set_condition(values['connectOutBoundSide_condition'])
        con_super.get_connect().\
            set_condition(values['connectParamOutbound_condition'])
        con_super.get_connect().\
            set_delayInMs(values['connectParamOutbound_delayInMs'])
      elif section == 'disconnectSupervision':
        ring_cadence.set_active(values['ringCadenceDetector_active'])
        ring_cadence.\
            set_ringCadenceOffDelayMs(values['ringCadenceDetector_OffDelayMs'])
      else:
        # Get config attribute method
        method = getattr(fxoConfig, "get_%s" % section)
        for attribute, value in values.items():
          attributeMethod = getattr(method(),"set_%s" % attribute)
          attributeMethod(value)   

    # Write current config to file using default file location
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  # Saves the new informations of the config.
  # All values are editable.
  @expose()
  def saveFxsConfig (self, **data):

    # Write current config to file using default file location
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  @expose()
  def addBoardFromEditor (self, **data):
    """Adds a board to the actual config from the editor"""

    log.info("Adding board: %s" % data)
    #TODO call the addBoard method of the model.

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  @expose()
  def deleteBoardFromEditor (self, **data):
    """Delete a board to the actual config from the editor"""

    log.info("Deleting board: %s" % data)

    #boardData = simplejson.loads(data['selectedBoard'])
    #if boardData['type'] == 'digital':
    #  for board in model.currentConfig.get_boards().get_sangomaDigitalBoard():
    #    if board.get_ID() == boardData['id']:
    #      model.deleteObjectFromEditor(board)
    #      del board
    #
    #else:  # Analog boards
    #  for board in model.currentConfig.get_boards().get_sangomaAnalogBoard():
    #    if board.get_ID() == boardData['id']:
    #      model.deleteObjectFromEditor(board)
    #      del board
    #
    ##save the config afterward
    #model.writeCurrentConfigToFile()
    #
    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  @expose()
  def addInterfaceFromEditor (self, **data):
    """Adds an interface to a board from the editor"""

    log.info("Adding interface: %s" % data)
    #TODO call the addInterface method of the model.

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  @expose()
  def deleteInterfaceFromEditor (self, **data):
    """Delete an interface from a board from the editor"""

    log.info("Deleting interface: %s" % data)
    #TODO call the deleteInterface method of the model.

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  @expose()
  def addPhysicalConfigFromEditor (self, **data):
    """Adds a physical config from the editor"""    
    log.info("Adding %s physical config" % data['configType'])
    if data['configType'] == 'T1':
      T1ConfigParams = {}
      T1ConfigParams['lineEncoding'] = 'B8ZS'
      T1ConfigParams['clocking'] = 'TERMINAL'
      T1ConfigParams['loopbackMode'] = 'DISABLED'
      T1ConfigParams['framing'] = 'ESF'
      T1ConfigParams['LBO'] = '0dB'
      
      #Create the audio parameters dictionary
      T1AudioParams = {}
      T1AudioParams['defaultPcmLaw'] = "PCMU"
      T1AudioParams['voicePacketLengthInMs'] = "20"
      T1AudioParams['toneSetName'] = "us"

      config = model.addT1SangomaDigitalConfig(model.currentConfig,
          'new_digital_T1_%s' % model.getNextPhysConfID(model.currentConfig),T1ConfigParams,T1AudioParams)
    elif data['configType'] == 'E1':
      E1ConfigParams = {}
      E1ConfigParams['lineEncoding'] = 'HDB3'
      E1ConfigParams['E1Signaling'] = 'CCS'
      E1ConfigParams['clocking'] = 'TERMINAL'
      E1ConfigParams['loopbackMode'] = 'DISABLED'
      E1ConfigParams['framing'] = 'CRC4'
      E1ConfigParams['E1Termination'] = '120ohm'

      #Create the audio parameters dictionary
      E1AudioParams = {}
      E1AudioParams['defaultPcmLaw'] = "PCMA"
      E1AudioParams['voicePacketLengthInMs'] = "20"
      E1AudioParams['toneSetName'] = "us"

      config = model.addE1SangomaDigitalConfig(model.currentConfig,
          'new_digital_E1_%s' % model.getNextPhysConfID(model.currentConfig),E1ConfigParams,E1AudioParams)
    elif data['configType'] == 'Analog':
      AudioParams = {}
      AudioParams['defaultPcmLaw'] = 'PCMU'
      AudioParams['voicePacketLengthInMs'] = "20"
      AudioParams['toneSetName'] = "us"

      config = model.addSangomaAnalogConfig(model.currentConfig,
          'new_analog_%s' % model.getNextPhysConfID(model.currentConfig),'USA',AudioParams)

    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  @expose()
  def deletePhysicalConfigFromEditor (self, **data):
    """Deltes a physical config from the editor"""

    log.info("Deleting physical config %s" % data['physConfigs'])

    for selection in simplejson.loads(data['physConfigs']):
      config = model.getObjectFromEditorId(selection['nodeId'])
      if type(config) == model.sangomaDigitalConfig:
        model.currentConfig.get_physicalInterfaceConfigs().get_sangomaDigitalConfig().remove(config)
      elif type(config) == model.sangomaAnalogConfig:
        model.currentConfig.get_physicalInterfaceConfigs().get_sangomaAnalogConfig().remove(config)
      model.deleteObjectFromEditor(config)

    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  @expose()
  def addChannelGroup(self, **data):
    log.info("Adding Channel Group to config: %s" % data)
    channelData = simplejson.loads(data['channelGroup'])

    if channelData['direction'] == 'IN':
      model.addChannelGroup(model.currentConfig,
                            ID=channelData['id'],
                            direction=channelData['direction'])
    else:
      model.addChannelGroup(model.currentConfig,
                      ID=channelData['id'],
                      outboundHuntingScheme=channelData['outboundHuntingScheme'],
                      direction=channelData['direction'])

    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  @expose()
  def deleteChannelGroup(self, **data):
    log.info("Deleting Channel Group: %s" % data)
    for selection in simplejson.loads(data['channelGroups']):
      channelGroup = model.getObjectFromEditorId(selection['nodeId'])
      model.currentConfig.get_callControl().get_channelGroups().get_channelGroup().remove(channelGroup)
      model.deleteObjectFromEditor(channelGroup)

    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  @expose()
  def addIsdnConfig(self, **data):
    isdnData = simplejson.loads(data['isdnConfig'])

    if isdnData['isdnConfigChoice'] == 'FAS':
      model.addFasConfig(model.currentConfig,
          displayName='new_FAS_%s' % model.getNextISDNConfID(model.currentConfig),
          termination='TERMINAL',
          switchVariant='5ESS',
          BChannelNegotiation='EXCLUSIVE',
          initiateRestartProcedure='YES',
          inbandProgressTonesGeneration='AS-NEEDED',
          inbandProgressTonesIndicator='PROGRESS')
    elif isdnData['isdnConfigChoice'] == 'NFAS':
      model.addNfasGroup(model.currentConfig,
          displayName='new_NFAS_%s' % model.getNextISDNConfID(model.currentConfig),
          termination='TERMINAL',
          switchVariant='5ESS',
          BChannelNegotiation='EXCLUSIVE',
          initiateRestartProcedure='YES',
          inbandProgressTonesGeneration='AS-NEEDED',
          inbandProgressTonesIndicator='PROGRESS')

    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))

  ##########################################################################
  @expose()
  def deleteIsdnConfig(self, **data):
    for selection in simplejson.loads(data['isdnConfigs']):
      isdnConfig = model.getObjectFromEditorId(selection['nodeId'])
      if type(isdnConfig) == model.fasConfig:
        model.currentConfig.get_callControl().get_isdn().get_fasConfig().remove(isdnConfig)
      elif type(isdnConfig) == model.nfasGroup:
        model.currentConfig.get_callControl().get_isdn().get_nfasGroup().remove(isdnConfig)
      model.deleteObjectFromEditor(isdnConfig)

    #If it was the last analog config, erase the <analog></analgo> from the config
    if len(model.currentConfig.get_callControl().get_isdn().get_fasConfig()) <= 0 and \
      len(model.currentConfig.get_callControl().get_isdn().get_nfasGroup()) <= 0:
      model.currentConfig.get_callControl().set_isdn(None)
    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))


  ##########################################################################
  @expose()
  def addAnalogConfig(self, **data): 
    analogData = simplejson.loads(data['analogConfig'])

    if analogData['analogConfigChoice'] == 'FXO':
      configParams = {}
      configParams['displayName'] = 'new_FXO_%s' % model.getNextAnalogConfID(model.currentConfig)
      # TODO FINISH THIS
      model.addAnalogFxoConfig(model.currentConfig, configParams)
    # TODO => FXS
    model.writeCurrentConfigToFile()
    return(str(extjs.FormActionSubmitResponse(success=True)))
  ##########################################################################
  @expose()
  def deleteAnalogConfig(self, **data):

    log.info("Deleting analog config %s" % data['analogConfigs'])

    for selection in simplejson.loads(data['analogConfigs']):
      analogConfig = model.getObjectFromEditorId(selection['nodeId'])
      if type(analogConfig) == model.analogFxoConfig:
        model.currentConfig.get_callControl().get_analog().get_analogFxoConfig().remove(analogConfig)
      elif type(analogConfig) == model.get_analogFxsConfig:
        model.currentConfig.get_callControl().get_analog().get_analogFxsConfig().remove(analogConfig)
      model.deleteObjectFromEditor(analogConfig)

    #If it was the last analog config, erase the <analog></analgo> from the config
    if not len(model.currentConfig.get_callControl().get_analog().get_analogFxoConfig()):
      model.currentConfig.get_callControl().set_analog(None)

    model.writeCurrentConfigToFile()

    return(str(extjs.FormActionSubmitResponse(success=True)))


