
# import pstnconfig autogenerated module contents into 'model' module
from paraxip.gateway_ui_model.pstnconfig import *
# import PstnConfigHelpers module contents into 'model' module
from paraxip.gateway_ui_model.PstnConfigHelpers import *

import logging
import os
import netborderPyInfra
log = logging.getLogger("model")

# Public module attributes

# The current pstnConfig
currentConfig = None

# The PARAXIP_INSTALLDIR value
strInstallDir = os.getenv("PARAXIP_INSTALLDIR")
if strInstallDir is None:
  log.error("PARAXIP_INSTALLDIR environment variable not set. Cannot "\
            "determine pstn configuration file location")       

# The file location of thew PSTN config in XML format
strPstnConfigFileName = os.path.join(strInstallDir,"config/pstn-config.xml")    

# Debug flag to allow validation of the config each time it is written out
# Turbogears bool configuration property "model.validateOnWrite" (default: False)
from turbogears import config
validateOnWrite = config.get("netborder.oamWebServer.model.validateOnWrite",
                              False)

###############################################################################
def getPSTNConfigFileName():
  return strPstnConfigFileName

def PSTNConfigFileExists ():
  if os.path.isfile(getPSTNConfigFileName()):
    # PSTN config file already exists
    log.info("PSTN config file %s already exists", getPSTNConfigFileName() )
    return True
  else:
    log.info("PSTN config file %s does not already exist", getPSTNConfigFileName() )
    return False

###############################################################################
# Writes current config to file as XML at the default location
def writeCurrentConfigToFile():
  # 'w+b' is for write, truncate, binary

  # Make a backup of current file
  # TODO make number of rolling backup configurable. Currently set to 5.
  try: 
    bRv = netborderPyInfra.renameWithTimestamp(getPSTNConfigFileName(),5)
    if bRv == False:
      log.debug("Unable to make a backup copy of pstn "
                "configuration file (%s). This might be the file creation.",
                getPSTNConfigFileName())  
  except:
    log.error("Error: exception caught while making a backup of pstn "
              "configuration file %s ",getPSTNConfigFileName())
  
  # Now ready to overwrite file
  outfile = open(getPSTNConfigFileName(), 'w+b')
  writeConfigToFileAsXML(currentConfig, outfile)
  outfile.close()
  if validateOnWrite:
    import netborderPyXML
    validator = netborderPyXML.XmlRelaxngValidator()
    strSchemaFile = os.path.join(strInstallDir, 'pstnconfig/pstn-config.rng')
    validator.configure(strSchemaFile)
    if validator.validateXmlFile(getPSTNConfigFileName()):
      log.info('PSTN config written to file "%s" is valid with respect '
        'to the XML schema' % getPSTNConfigFileName())
    else:
      log.error('PSTN config written to file "%s" is NOT valid with respect '
        'to the XML schema, see paraxip log for details' % 
        getPSTNConfigFileName())
    


