
/*
 * Reload the treeview
 * @param reloadFromCurrentNode: true if we should reset focus to a specific node;
 *    false if not
 */
function reloadTree(reloadFromCurrentNode){
  var currentNode = treePanel.getSelectionModel().getSelectedNode(); //retrieve the current node
  treePanel.collapseAll();

  treePanel.root.reload(function(){
    if( reloadFromCurrentNode ) {
      goToNode( currentNode.id );
    } else {
      //Brings the emptyPanel view wich has nothing in it
      Ext.getCmp('content-panel').layout.setActiveItem(0);
    }
  });
};

/*
 * Bring the user to the node corresponding to the given id
 * @param id: id of the node to select
 */
function goToNode(id){
	var destinationNode = treePanel.getNodeById(id);
	var currentPath = destinationNode.getPath();
	var parentNode = destinationNode.parentNode;

  treePanel.expandPath(currentPath);
	//To render all the nodes, fire a click event on each parent
	//node until the root node is reached<
	if( !parentNode.isRoot ) {
		treePanel.fireEvent('click',parentNode); //simulate a click event to reload the form
	}
  treePanel.fireEvent('click',destinationNode); //simulate a click event to reload the form
  treePanel.getSelectionModel().select(destinationNode);
};


// This is an empty panel only used to manage the undefined click and the reloads of
// the tree. It could eventually contain a helper message to tell the client what it
// is possible to accomplish in the editor.
var emptyPanel = new Ext.Panel({
	id: 'emtpy-panel',
	layout: 'absolute',
  autoLoad: '/static/editor_first_panel.html'
});
// This is the main content center region that will contain each example layout panel.
// It will be implemented as a CardLayout since it will contain multiple panels with
// only one being visible at any given time.

var contentPanel = {
  id: 'content-panel',
  region: 'center', // this is what makes this panel into a region within the containing layout
  layout: 'card',
	layoutConfig : {
		deferredRender: true,
		layoutOnTabChange: true
	},
	defaults: {
		hideMode: 'offsets'
	},
  margins: '2 5 5 0',
  activeItem: 0,
  border: false,
  items: [
    emptyPanel,
    gridBoards,
    panelDigitalBoard,
    panelAnalogBoard,
    panelVirtualDigitalBoard,
    frmDigitalInterface,
    frmAnalogInterface,
    frmVirtualDigitalInterface,
    gridPhysicalConfig,
    frmDigitalT1PhysConf,
    frmDigitalE1PhysConf,
    frmAnalogPhysConf,
    gridChannelGroup,
    frmChannelGroup,
    gridISDNConfiguration,
    frmIsdnFas,
    frmIsdnNfas,
    gridAnalogConfiguration,
    frmAnalogFxo    
  ]
};

// Go ahead and create the TreePanel now so that we can use it below
var treePanel = new Ext.tree.TreePanel({
  id            : 'tree-panel',
  region        : 'west',
	layout				: 'absolute',
	margins       : '2 2 0 2',
  // enableDD      : true,
  width         : 175,
  minSize       : 100,
  maxSize       : 250,
  // tree-specific configs:
  rootVisible   : false,
  lines: false,
  singleExpand: false,
  useArrows: true,

  loader: new Ext.tree.TreeLoader({
            dataUrl   :'${fetch}',
            preloadChildren  : true
          }),

  root: new Ext.tree.AsyncTreeNode()
});

// Assign the changeLayout function to be called on tree node click.
// "n" is newly clicked-on node
treePanel.on('click', function(n){
  var sn = this.selModel.selNode || {}; // selNode is null on initial selection
  if(n.id != sn.id){

    // ignore clicks on currently selected node
    // if the form is defined we show it
    if(n.attributes.frm){
      // load the form and send the node id as a parameter

        //console.info('Component type :',Ext.getCmp(n.attributes.frm).getXType());
      if(n.attributes.frm == 'gridBoards') {
        Ext.getCmp('gridBoards').getStore().load({
          //url: 'getGridBoardsData',
          params: {'id':n.id}
        });
      }
      if(n.attributes.frm == 'panelAnalogBoard') {
        Ext.getCmp('frmAnalogBoard').load({
          url: 'config/getFrmAnalogBoardData',
          params: {'id':n.id}
        });
        Ext.getCmp('gridInterfacesAnalog').getStore().load({
           //url: 'getGridInterfacesData',
           params: {'id':n.id}
        });
      }
      if(n.attributes.frm == 'panelDigitalBoard') {
        Ext.getCmp('frmDigitalBoard').load({
          url: 'config/getFrmDigitalBoardData',
          params: {'id':n.id}
        });
        Ext.getCmp('gridInterfacesDigital').getStore().load({
           //url: 'getGridInterfacesData',
           params: {'id':n.id}
        });
      }
      if(n.attributes.frm == 'panelVirtualDigitalBoard') {
        Ext.getCmp('frmVirtualDigitalBoard').load({
          url: 'config/getFrmVirtualDigitalBoardData',
          params: {'id':n.id}
        });
      }
      if(n.attributes.frm == 'frmAnalogInterface') {
        Ext.getCmp('frmAnalogInterface').load({
          url: 'config/getFrmAnalogInterfaceData',
          params: {'id':n.id}
        });
      }
      if(n.attributes.frm == 'frmDigitalInterface') {
        Ext.getCmp('frmDigitalInterface').load({
          url: 'config/getFrmDigitalInterfaceData',
          params: {'id':n.id}
        });
      }
      if(n.attributes.frm == 'frmVirtualDigitalInterface') {
        Ext.getCmp('frmVirtualDigitalInterface').load({
          url: 'config/getFrmVirtualDigitalInterfaceData',
          params: {'id':n.id},
          success: function(){
						levelControlVisibleFields.call(Ext.getCmp('frmDigitalT1PhysConf'));
						if( Ext.getCmp('mtu_sizeT1_text').rendered ) {
							isDChannelEnabled.call(Ext.getCmp('disableDChannelT1'));
						}
					}
        });
      }
      if(n.attributes.frm == 'frmDigitalT1PhysConf') {
        Ext.getCmp('frmDigitalT1PhysConf').load({
          url: 'config/getFrmDigitalT1PhysConfData',
          params: {'id':n.id},
          success: function(){
						levelControlVisibleFields.call(Ext.getCmp('frmDigitalT1PhysConf'));
						if( Ext.getCmp('mtu_sizeT1_text').rendered ) {
							isDChannelEnabled.call(Ext.getCmp('disableDChannelT1'));
						}
					}
        });
      }
      if(n.attributes.frm == 'frmDigitalE1PhysConf') {
        Ext.getCmp('frmDigitalE1PhysConf').load({
          url: 'config/getFrmDigitalE1PhysConfData',
          params: {'id':n.id},
          scope: Ext.getCmp('frmDigitalE1PhysConf'),
						success: function(){
							levelControlVisibleFields.call(Ext.getCmp('frmDigitalE1PhysConf'));
							if( Ext.getCmp('mtu_sizeE1_text').rendered ) {
								isDChannelEnabled.call(Ext.getCmp('disableDChannelE1'));
							}
						}
        });
      }
      if(n.attributes.frm == 'frmAnalogPhysConf') {
        Ext.getCmp('frmAnalogPhysConf').load({
          url: 'config/getFrmAnalogPhysConfData',
          params: {'id':n.id},
          scope: Ext.getCmp('frmAnalogPhysConf'),
          success: levelControlVisibleFields
        });
      }
      if(n.attributes.frm == 'gridPhysicalConfig') {
        Ext.getCmp('gridPhysicalConfig').getStore().load({
          params: {'id':n.id}
        });
      }
      if(n.attributes.frm == 'gridChannelGroup') {
        Ext.getCmp('gridChannelGroup').getStore().load({
          params: {'id':n.id}
        });
      }
      if(n.attributes.frm == 'frmChannelGroup') {
        Ext.getCmp('frmChannelGroup').load({
          url: 'config/getFrmChannelGroupData',
          params: {'id':n.id},
					success: showOutboundScheme
        });
      }
      if(n.attributes.frm == 'gridISDNConfiguration') {
        Ext.getCmp('gridISDNConfiguration').getStore().load({
          params: {'id':n.id}
        });
				//load grid in case it hasn't been loaded yet.
        Ext.getCmp('gridBoards').getStore().load({
          //url: 'getGridBoardsData',
          params: {'id':n.id}
        });
      }
      if(n.attributes.frm == 'frmIsdnFas') {
        Ext.getCmp('frmIsdnFas').load({
          url: 'config/getFrmIsdnFasData',
          params: {'id':n.id}
        });
      }
      if(n.attributes.frm == 'frmIsdnNfas') {
        Ext.getCmp('frmIsdnNfas').load({
          url: 'config/getFrmIsdnNfasData',
          params: {'id':n.id}
        });
      }
      if(n.attributes.frm == 'gridAnalogConfiguration') {
        Ext.getCmp('gridAnalogConfiguration').getStore().load({
          params: {'id':n.id}
        });
				//load grid in case it hasn't been loaded yet.
        Ext.getCmp('gridBoards').getStore().load({
          //url: 'getGridBoardsData',
          params: {'id':n.id}
        });
      }
      if(n.attributes.frm == 'frmAnalogFxo') {
        Ext.getCmp('frmAnalogFxo').load({
          url: 'config/getFrmAnalogFxoData',
          params: {'id':n.id},
          success: isNumberofRingsVisible
        });
      }
      // show form
      Ext.getCmp('content-panel').layout.setActiveItem(n.attributes.frm);
    }
    else{
      Ext.getCmp('content-panel').layout.setActiveItem(0);
    }
  }
});


// this fixes the problem of the static load from an ajax source
// see http://extjs.com/forum/showthread.php?t=34902

Ext.tree.TreeLoader.override({
  processResponse : function(response, node, callback){
      var json = response.responseText;
      try {
          var o = eval('('+json+')');
          node.beginUpdate();
          for(var i = 0, len = o.length; i < len; i++){
              var n = this.createNode(o[i]);
              if(n){
                  node.appendChild(n);
                  if(this.preloadChildren){
                      this.doPreload(n);
                  }
              }
          }
          node.endUpdate();
          if(typeof callback == 'function'){
              callback(this, node);
          }
      }catch(e){
          this.handleFailure(response);
      }
  }
});


// Finally, build the main layout once all the pieces are ready.  This is also a good
// example of putting together a full-screen BorderLayout within a Panel.
var editorMainPanel = new Ext.Panel({
  layout: 'border',
  title: 'PSTN Config',
	hideMode: 'offsets',
  //renderTo: '$renderTo',
  //height: 570,
  //width: 754,
  defaults:{
    autoScroll: true,
    style: 'text-align:left',
		hideMode: 'offsets'
	},
  items: [treePanel, contentPanel]
});




