/*
 * Paraxip overrides to Ext JS Library 2.2
 * Copyright(c) 2008, Paraxip Technologies
 */

/*
  Ext.form.Field override to allow hiding the whole container, including
    the associated dynamic label. Use once layout is done for the field.
  From "timb" at http://extjs.com/forum/showthread.php?t=17396
*/

// turn on validation errors beside the field globally
Ext.form.Field.prototype.msgTarget = 'side';
Ext.form.Field.prototype.anchor = '60%';
// to enable quicktips
Ext.QuickTips.init();

Ext.override(Ext.form.Field, {

  showContainer: function() {
      this.enable();
      this.show();
      // show entire container and children (including label if applicable)
      this.getEl().up('.x-form-item').setDisplayed(true);
  },

  hideContainer: function() {
      this.disable(); // for validation
      this.hide();
      // hide container and children (including label if applicable)
      this.getEl().up('.x-form-item').setDisplayed(false);
  },

  setContainerVisible: function(visible) {
      if (visible) {
          this.showContainer();
      } else {
          this.hideContainer();
      }
      return this;
  },

  alignErrorIcon : function(){
    if (this.helpImage){
      this.errorIcon.alignTo(this.el, 'tl-tr', [20, 0]);
    }else{
      this.errorIcon.alignTo(this.el, 'tl-tr', [2, 0]);
    }
  },

  alignHelpImage : function(){
    this.helpImage.alignTo(this.el, 'tl-tr', [2, 0]);
  },

  afterRender : function() {
    var label = findLabel(this);

    if(label && this.helpText)
    {
      var elp = this.el.findParent('.x-form-element', 5, true);
      this.helpImage = elp.createChild({
          //tag: 'img',
         /* src: '/static/images/help.gif',
          style: 'position:absolute; left:0;top:0;display:block;',
          width: 16,
          height: 16*/
          style : 'width:16px;height:18px;position:absolute;left:0;top:0;display:block;background:transparent url(/static/images/help.gif) no-repeat 0 2px;cursor:pointer!important;'
        });
        //this.helpImage.alignTo(this.el, 'tl-tr', [2, 0]);
        //this.on('resize',function(){
        //          this.helpImage.alignTo(this.el, 'tl-tr', [2, 0]);
        //          }, this);
        this.alignHelpImage();
        this.on('resize', this.alignHelpImage, this);


      this.helpImage.on("click", function()
      {

        //this.ownerCt.add(helpWindow);
        helpWindow.setTitle(this.fieldLabel + " Help");
        helpWindow.show();

        var textarea = Ext.getDom("helpText_area");
        textarea.value = Paraxip.doc.
            pstnConfigNodeDocText[this.helpText];
      }, this);//scope
    }

    Ext.form.Field.superclass.afterRender.call(this);
    this.initEvents();
    this.initValue();
  },

  findParentBy: function(fn) {
    var p = this;
    do {
      p = p.ownerCt;
    } while (p !== null && !fn(p));
    return p;
  },

  findParentByType: function(xtype) {
    return typeof xtype == 'function' ?
           this.findParentBy(function(p) {
             return p.constructor === xtype;
           }) :
           this.findParentBy(function(p) {
             var type = p.constructor.xtype;
             return type === xtype;
           });
  }

});

// Override ComboBox for the help and error
Ext.override(Ext.form.ComboBox, {

  alignErrorIcon : function(){
    if (this.helpImage){
      this.errorIcon.alignTo(this.wrap, 'tl-tr', [20, 0]);
    }else{
      this.errorIcon.alignTo(this.wrap, 'tl-tr', [2, 0]);
    }
  },

  alignHelpImage : function(){
    this.helpImage.alignTo(this.wrap, 'tl-tr', [2, 0]);
  }
});

// this fixes the problem of the static load from an ajax source
// see http://extjs.com/forum/showthread.php?t=34902

Ext.override( Ext.tree.TreeLoader, {
  processResponse : function(response, node, callback){
      var json = response.responseText;
      try {
          var o = eval('('+json+')');
          node.beginUpdate();
          for(var i = 0, len = o.length; i < len; i++){
              var n = this.createNode(o[i]);
              if(n){
                  node.appendChild(n);
                  if(this.preloadChildren){
                      this.doPreload(n);
                  }
              }
          }
          node.endUpdate();
          if(typeof callback == 'function'){
              callback(this, node);
          }
      }catch(e){
          this.handleFailure(response);
      }
  }
});

/* Ext.Container
 *  getItemsValueAsJson()
 *    Recurse thru "items" and return their names and native values as JSON, ex.
        {digitalT1: {framing: 'ESF',
                     lineEncoding: 'AMI'},
         audio: {echo: true,
                 tap: 2}}
 *
 */
Ext.namespace('Paraxip'); // Ensure Paraxip namespace is defined
Paraxip.getItemsValue = function(items) {
  var itemsValues = {};
  items.each(function(item) {
    if(item.items === undefined) {
      // Simple item (field): getValue for Ext.form.Field subclasses return
      //  native object (int, float, etc.)
      if( !item.disabled ) {
        itemsValues[item.name] = item.getValue();
      }
    } else {
      // The item itself contains items - recursive call
      itemsValues[item.name] = Paraxip.getItemsValue(item.items);
    }
  });
  return itemsValues;
};

Ext.override(Ext.Container, {
  getItemsValueAsJson: function() {
    return Ext.util.JSON.encode(Paraxip.getItemsValue(this.items));
  }
});

var helpWindow = new Ext.Window({
  id          : "helpWindow",
  layout      : 'border',
  width       : 505,
  height      : 150,
  closeAction : 'hide',
  plain       : true
});

/*
 * Field Override for text files
 *
 * */
var helpTextArea= new Ext.form.TextArea({
  id: 'helpText_area',
  valueField: 'helpText',
  name: 'helpText',
  hideLabel: true,
  style : "height: 113px; width: 482px;",
  region:'center',
//  grow: true,
  readOnly    : true
});

helpWindow.add(helpTextArea);


var findLabel = function(field) {

    var wrapDiv = null;
    var label = null;

    //find form-item and label
    wrapDiv = field.getEl().up('div.x-form-item');
    if(wrapDiv)
    {
        label = wrapDiv.child('label');
    }
    if(label) {
        return label;
    }
};

