/*
 * Paraxip Oam Common Library to Ext JS Library 2.2
 * Copyright(c) 2008, Paraxip Technologies
 */

Ext.namespace('Paraxip');
/*
 * Those methods scans the boardGrid and for each board, test if it is an analog board,
 * a digital board or a virtual_digital board. Depending on the method, it returns true
 * if one of the boards matches the disired type. To test the board type, we use the 
 * isBoardAnalog, isBoardDigital and isBoardVirtualDigital methods.
 */
Paraxip.gridHasAnalogBoard = function(grid) {
  var records = grid.getStore().getRange();
  var board_type;

  if (records.length > 0) {
    for (var i = 0; i < records.length; i++) {
      board_type = records[i].get('model');
      if ( Paraxip.isBoardAnalog(board_type) ) {
        return true;
      }
    }
  }
  return false;
};

Paraxip.gridHasDigitalBoard = function(grid) {
	var records = grid.getStore().getRange();
  var board_type;

  if (records.length > 0) {
    for (var i = 0; i < records.length; i++) {
      board_type = records[i].get('model');
      if ( Paraxip.isBoardDigital(board_type) ) {
        return true;
      }
    }
  }
  return false;
};

Paraxip.gridHasVirtualDigitalBoard = function(grid) {
  var records = grid.getStore().getRange();
  var board_type;

  if (records.length > 0) {
    for (var i = 0; i < records.length; i++) {
      board_type = records[i].get('model');
      if ( Paraxip.isBoardVirtualDigital(board_type) ) {
        return true;
      }
    }
  }
  return false;
};

//If selected board is an A200 or an A400, it is analog.
Paraxip.isBoardAnalog = function(boardType) {
  return ( boardType == "A200" || boardType == "A400" );
};
// Digital boards starts with "A1"
Paraxip.isBoardDigital = function(boardType) {
  return (/^A1/).test(boardType);
};

Paraxip.isBoardVirtualDigital = function(boardType) {
  return (boardType == "VIRTUAL_DIGITAL");
};

//gateway status list
Paraxip.activeStatus = {
	text: "Gateway Status:  started",
	iconCls: ''
};

Paraxip.downStatus = {
	text: "Gateway Status:  stopped",
	iconCls: ''
};

Paraxip.workingStatus = {
	text: "Gateway Status:  working"
};

/*
 * Change the Gateway Status: xxxx value in the status bar of the main panel
 * @param status: the status to display;
 * @param isStartDisabled: true if the start button should be grayed out; false if not;
 * @param isStopDisabled: true if the stop button should be grayed out; false if not;
 */
Paraxip.changeGwStatusBar = function(status, isStartDisabled, isStopDisabled) {
	Ext.getCmp('startGwButton').setDisabled(isStartDisabled);
	Ext.getCmp('stopGwButton').setDisabled(isStopDisabled);
	if( status == Paraxip.workingStatus) {
		Ext.getCmp('gw_status_bar').showBusy(status);
	} else {
		Ext.getCmp('gw_status_bar').setStatus(status);
	}
};

/* 
 * Show a progress bar freezing all the ui until the request is finished.
 * @param msg: the message to display in the message window.
 */
Paraxip.showProgressBar = function(msg){
	Ext.MessageBox.show({
	 msg: msg,
	 progressText: 'Starting...',
	 width:300,
	 wait:true,
	 waitConfig: {interval:200},
	 //icon:'ext-mb-download', //custom class in msg-box.html
	 animEl: 'mb7'
 })
}

/*
 * Hide the progress bar window showed with the method Paraxip.showProgressBar
 */
Paraxip.hideProgressBar = function(){
	Ext.MessageBox.hide();
}

/*
 * Display a pop-up message telling the user his last operation failed.
 */
Paraxip.failureMessage = function(){
  Ext.Msg.alert("Error!","Your last operation failed. Please contact support if the problem persist.");
};



